/**
 * 
 * TimePicker 
 * Author: Eric Stevens
 *  
 * Note: AMPM is not fully implemented
 *
 * Based on code from Clockface
 * Author: Vitaliy Potapov
 * Project page: http://github.com/vitalets/clockface
 * Copyright (c) 2012 Vitaliy Potapov. Released under MIT License.
 *
**/

(function ($) {

    var Timepicker = function (element, options) {
        this.$element = $(element);
        this.options = $.extend({}, $.fn.timepicker.defaults, options, this.$element.data());
        this.init();
    };

    Timepicker.prototype = {
        
        constructor: Timepicker,
        
        init: function () {

            //apply template
            this.$timepicker = $($.fn.timepicker.template);
            this.$timepicker.hide();

            //this.$ampm = this.$timepicker.find('.ampm');

            this.$hours = this.$timepicker.find('.timepicker-hours td');
            this.$minutes = this.$timepicker.find('.timepicker-minutes td');
            
            // ml
            this.$timepicker.find('.timepicker-now').text(this.options.now);
            this.$timepicker.find('.timepicker-clear').text(this.options.clear);
            this.$timepicker.find('.timepicker-header-hour').text(this.options.hour);
            this.$timepicker.find('.timepicker-header-minute').text(this.options.minute);

            //internal vars
            this.ampm = null;
            this.hour = null;
            this.minute = null;

            this.top_bottom_pos = 'below';

            //click am/pm 
            //this.$ampm.click($.proxy(this.clickAmPm, this));

            //click cell
            this.$timepicker.on('click', '.timepicker-hour', $.proxy(this.clickHour, this));
            this.$timepicker.on('click', '.timepicker-minute', $.proxy(this.clickMinute, this));

            this.$timepicker.on('click', '.timepicker-now', $.proxy(this.clickNow, this));
            this.$timepicker.on('click', '.timepicker-clear', $.proxy(this.clickClear, this));

            this.parseFormat();
            this.prepareRegexp();

            //set ampm text
            //this.ampmtext = this.is24 ? { am: '12-23', pm: '0-11' } : { am: 'AM', pm: 'PM' };


            this.$timepicker.addClass('dropdown-menu').appendTo('body');
            if (this.options.trigger === 'focus') {
                this.$element.on('focus.timepicker', $.proxy(function (e) { this.show(); }, this));
            }

            // Click outside hide it. Register single handler for all timepicker widgets
            $(document).off('click.timepicker').on('click.timepicker', $.proxy(function (e) {
                var $target = $(e.target);
                //click inside some timepicker --> do nothing
                if ($target.closest('.timepicker').length) {
                    return;
                }
                //iterate all open timepicker and close all except current
                $('.timepicker-open').each(function () {
                    if (this === e.target) {
                        return;
                    }
                    $(this).timepicker('hide');
                });
            }, this));

        },

        show: function (value) {
            if (this.$timepicker.is(':visible')) {
                return;
            }
            if (value === undefined) {
                value = this.$element.val();
            }

            // top or bottom
            var pickerHeight = this.$timepicker.outerHeight();
            var scrollTop = $(window).scrollTop();
            var windowHeight = $(window).height();
            var offset = this.$element.offset();
            var height = this.$element.outerHeight(false);
            var top_overflow = -scrollTop + offset.top - pickerHeight;
            var bottom_overflow = scrollTop + windowHeight - (offset.top + height + pickerHeight);
            if (Math.max(top_overflow, bottom_overflow) === bottom_overflow) {
                // we need to appear below
                this.$timepicker.addClass("bottom");
                this.$timepicker.removeClass("top");
                this.top_bottom_pos = 'below';
            } else {
                // we need to appear above
                this.$timepicker.addClass("top");
                this.$timepicker.removeClass("bottom");
                this.top_bottom_pos = 'above';
            }

            this.$element.addClass('timepicker-open');
            this.$element.on('keydown.timepicker', $.proxy(this.keydown, this));
            this.setLocation();
            $(window).on('resize.timepicker', $.proxy(this.setLocation, this));

            this.$timepicker.show();
            this.setTime(value);

            //trigger shown event
            this.$element.triggerHandler('shown.timepicker', this.getTime(true));
        },


        hide: function () {
            this.$timepicker.hide();
            this.$element.removeClass('timepicker-open');
            this.$element.off('keydown.timepicker');
            $(window).off('resize.timepicker');

            //trigger hidden event
            this.$element.triggerHandler('hidden.timepicker', this.getTime(true));
        },

        toggle: function (value) {
            if (this.$timepicker.is(':visible')) {
                this.hide();
            } else {
                this.show(value);
            }
        },

        setTime: function (value) {

            var res, hour, minute, ampm = 'am';

            //no new value 
            if (value === undefined) {
                //if ampm null, it;s first showw, need to render hours ('am' by default)
                if (this.ampm === null) {
                    this.setAmPm('am');
                }
                return;
            }

            //take value from Date object
            if (value instanceof Date) {
                hour = value.getHours();
                minute = value.getMinutes();
            }

            //parse value from string
            if (typeof value === 'string' && value.length) {
                res = this.parseTime(value);

                //'24' always '0'
                if (res.hour === 24) {
                    res.hour = 0;
                }

                hour = res.hour;
                minute = res.minute;
                ampm = res.ampm;
            }

            //try to set ampm automatically
            if (hour > 11 && hour < 24) {
                ampm = 'pm';
                //for 12h format substract 12 from value
                if (!this.is24 && hour > 12) {
                    hour -= 12;
                }
            } else if (hour >= 0 && hour < 11) {
                //always set am for 24h and for '0' in 12h
                if (this.is24 || hour === 0) {
                    ampm = 'am';
                }
                //otherwise ampm should be defined in value itself and retrieved when parsing
            }

            this.setAmPm(ampm);
            this.setHour(hour);
            this.setMinute(minute);
        },

        setAmPm: function (value) {
            if (value === this.ampm) {
                return;
            } else {
                this.ampm = value === 'am' ? 'am' : 'pm';
            }

            //set link's text
            //this.$ampm.text(this.ampmtext[this.ampm]);

            this.refreshHighlight();
        },

        setHour: function (value) {
            value = parseInt(value, 10);
            value = isNaN(value) ? null : value;
            if (value < 0 || value > 23) {
                value = null;
            }

            if (value === this.hour) {
                return;
            } else {
                this.hour = value;
            }

            this.refreshHighlight();
        },

        setMinute: function (value) {
            value = parseInt(value, 10);
            value = isNaN(value) ? null : value;
            if (value < 0 || value > 59) {
                value = null;
            }

            if (value === this.minute) {
                return;
            } else {
                this.minute = value;
            }

            this.refreshHighlight();
        },

        refreshHighlight: function () {

            var i, cell;

            this.$hours.removeClass('active');
            this.$minutes.removeClass('active');

            if (this.hour != null && this.hour >= 0 && this.hour < 24) {
                for (i=0; i< this.$hours.length; i++) {
                    cell = $(this.$hours[i]);
                    if (cell.text() == this.hour) {
                        cell.addClass('active');
                    }
                }
            }

            if (this.minute != null && this.minute >= 0 && this.minute < 60) {
                for (i=0; i< this.$minutes.length; i++) {
                    cell = $(this.$minutes[i]);
                    if (cell.text() == this.minute) {
                        cell.addClass('active');
                    }
                }
            }

        },



        clickHour: function (e) {

            var $target = $(e.target);
            var value = $target.hasClass('active') ? null : $target.text();

            this.setHour(value);

            this.$element.val(this.getTime()).change();

            //trigger pick event
            this.$element.triggerHandler('pick.timepicker', this.getTime(true));

        },

        clickMinute: function (e) {

            var $target = $(e.target);
            var value = $target.hasClass('active') ? null : $target.text();

            this.setMinute(value);

            this.$element.val(this.getTime()).change();

            //trigger pick event
            this.$element.triggerHandler('pick.timepicker', this.getTime(true));

            // auto-hide
            if (this.hour != null && this.minute != null) {
                this.hide();
            }

        },

        clickAmPm: function (e) {

            e.preventDefault();

            //toggle am/pm
            this.setAmPm(this.ampm === 'am' ? 'pm' : 'am');

            //trigger pick event
            this.$element.triggerHandler('pick.timepicker', this.getTime(true));
        },

        clickNow: function (e) {

            this.setHour(new Date().getHours());
            this.setMinute(new Date().getMinutes());

            this.$element.val(this.getTime()).change();

            //trigger pick event
            this.$element.triggerHandler('pick.timepicker', this.getTime(true));

            this.hide();
        },

        clickClear: function (e) {

            this.setHour(null);
            this.setMinute(null);

            this.$element.val(this.getTime()).change();

            //trigger pick event
            this.$element.triggerHandler('pick.timepicker', this.getTime(true));

            this.hide();

        },

        setLocation: function () {
            var zIndex = parseInt(this.$element.parents().filter(function () {
                return $(this).css('z-index') != 'auto';
            }).first().css('z-index'), 10) + 10;

            var offset = this.$element.offset();

            var top = (this.top_bottom_pos == 'below') ?
                offset.top + this.$element.outerHeight() :
                offset.top - this.$timepicker.outerHeight();

            this.$timepicker.css({
                top: top,
                left: offset.left,
                zIndex: zIndex
            });

        },

        keydown: function (e) {
            //tab, escape, enter --> hide
            if (/^(9|27|13)$/.test(e.which)) {
                this.hide();
                return;
            }

            clearTimeout(this.timer);
            this.timer = setTimeout($.proxy(function () {
                this.setTime(this.$element.val());
            }, this), 500);
        },

        parseFormat: function () {
            var format = this.options.format,
                hFormat = 'HH',
                mFormat = 'mm';

            //hour format    
            $.each(['HH', 'hh', 'H', 'h'], function (i, f) {
                if (format.indexOf(f) !== -1) {
                    hFormat = f;
                    return false;
                }
            });

            //minute format
            $.each(['mm', 'm'], function (i, f) {
                if (format.indexOf(f) !== -1) {
                    mFormat = f;
                    return false;
                }
            });

            //is 24 hour format
            this.is24 = hFormat.indexOf('H') !== -1;

            this.hFormat = hFormat;
            this.mFormat = mFormat;
        },

        parseTime: function (value) {
            var hour = null,
                minute = null,
                ampm = 'am',
                parts = [], digits;

            value = $.trim(value);

            //try parse time from string assuming separator exist
            if (this.regexpSep) {
                parts = value.match(this.regexpSep);
            }

            if (parts && parts.length) {
                hour = parts[1] ? parseInt(parts[1], 10) : null;
                minute = parts[2] ? parseInt(parts[2], 10) : null;
                ampm = (!parts[3] || parts[3].toLowerCase() === 'a') ? 'am' : 'pm';
            } else {
                //if parse with separator failed, search for 1,4-digit block and process it
                //use reversed string to start from end (usefull with full dates)
                //see http://stackoverflow.com/questions/141348/what-is-the-best-way-to-parse-a-time-into-a-date-object-from-user-input-in-javas
                value = value.split('').reverse().join('').replace(/\s/g, '');
                parts = value.match(this.regexpNoSep);
                if (parts && parts.length) {
                    ampm = (!parts[1] || parts[1].toLowerCase() === 'a') ? 'am' : 'pm';
                    //reverse back
                    digits = parts[2].split('').reverse().join('');
                    //use smart analyzing to detect hours and minutes
                    switch (digits.length) {
                        case 1:
                            hour = parseInt(digits, 10); //e.g. '6'
                            break;
                        case 2:
                            hour = parseInt(digits, 10); //e.g. '16'
                            //if((this.is24 && hour > 24) || (!this.is24 && hour > 12)) { //e.g. 26
                            if (hour > 24) { //e.g. 26
                                hour = parseInt(digits[0], 10);
                                minute = parseInt(digits[1], 10);
                            }
                            break;
                        case 3:
                            hour = parseInt(digits[0], 10);  //e.g. 105
                            minute = parseInt(digits[1] + digits[2], 10);
                            if (minute > 59) {
                                hour = parseInt(digits[0] + digits[1], 10); //e.g. 195
                                minute = parseInt(digits[2], 10);
                                if (hour > 24) {
                                    hour = null;
                                    minute = null;
                                }
                            }
                            break;
                        case 4:
                            hour = parseInt(digits[0] + digits[1], 10); //e.g. 2006
                            minute = parseInt(digits[2] + digits[3], 10);
                            if (hour > 24) {
                                hour = null;
                            }
                            if (minute > 59) {
                                minute = null;
                            }
                    }
                }
            }

            return { hour: hour, minute: minute, ampm: ampm };
        },

        prepareRegexp: function () {
            //take separator from format
            var sep = this.options.format.match(/h\s*([^hm]?)\s*m/i); //HH-mm, HH:mm
            if (sep && sep.length) {
                sep = sep[1];
            }

            //sep can be null for HH, and '' for HHmm 
            this.separator = sep;

            //parse from string
            //use reversed string and regexp to parse 2-digit minutes first
            //see http://stackoverflow.com/questions/141348/what-is-the-best-way-to-parse-a-time-into-a-date-object-from-user-input-in-javas
            //this.regexp = new RegExp('(a|p)?\\s*((\\d\\d?)' + sep + ')?(\\d\\d?)', 'i');

            //regexp, used with separator
            this.regexpSep = (this.separator && this.separator.length) ? new RegExp('(\\d\\d?)\\s*\\' + this.separator + '\\s*(\\d?\\d?)\\s*(a|p)?', 'i') : null;

            //second regexp applied if previous has no result or separator is empty (to reversed string)
            this.regexpNoSep = new RegExp('(a|p)?\\s*(\\d{1,4})', 'i');
        },


        getTime: function (asObject) {
            if (asObject === true) {
                return {
                    hour: this.hour,
                    minute: this.minute,
                    ampm: this.ampm
                };
            }

            var hour = this.hour !== null ? this.hour + '' : '',
                minute = this.minute !== null ? this.minute + '' : '',
                result = this.options.format;

            if (!hour.length && !minute.length) {
                return '';
            }

            if (this.hFormat.length > 1 && hour.length === 1) {
                hour = '0' + hour;
            }

            if (this.mFormat.length > 1 && minute.length === 1) {
                minute = '0' + minute;
            }

            //delete separator if no minutes
            if (!minute.length && this.separator) {
                result = result.replace(this.separator, '');
            }

            result = result.replace(this.hFormat, hour).replace(this.mFormat, minute);
            if (!this.is24) {
                if (result.indexOf('A') !== -1) {
                    result = result.replace('A', this.ampm.toUpperCase());
                } else {
                    result = result.replace('a', this.ampm);
                }
            }

            return result;
        },

        destroy: function () {
            this.hide();
            this.$timepicker.remove();
            if (this.options.trigger === 'focus') {
                this.$element.off('focus.timepicker');
            }
        }
    };




    $.fn.timepicker = function (option) {

        var d, args = Array.apply(null, arguments);
        args.shift();

        //getTime returns string (not jQuery onject)
        if (option === 'getTime' && this.length && (d = this.eq(0).data('timepicker'))) {
            return d.getTime.apply(d, args);
        }

        return this.each(function () {
            var $this = $(this),
                data = $this.data('timepicker'),
                options = typeof option == 'object' && option;
            if (!data) {
                $this.data('timepicker', (data = new Timepicker(this, options)));
            }
            if (typeof option == 'string' && typeof data[option] == 'function') {
                data[option].apply(data, args);
            }
        });
    };

    $.fn.timepicker.defaults = {

        format: 'HH:mm',  // http://momentjs.com/docs/#/displaying/format/
        trigger: 'focus',  // focus|manual
        now: "Now",
        clear: "Clear",
        hour: "Hour",
        minute: "Minute"
    };


    $.fn.timepicker.template = '' +
        '<div class="timepicker-popover">' +
            '<div class="arrow"></div>' +
            '<div class="popover-content timepicker">' +
                '<table class="timepicker-picker">' +
                    '<tr>' +
                        '<td class="timepicker-header timepicker-header-hour">' +
                            'Hour' +
                        '</td>' +
                        '<td></td>' +
                        '<td class="timepicker-header timepicker-header-minute">' +
                            'Minute' +
                        '</td>' +
                    '</tr>' +
                    '<tr>' +
                        '<td>' +
                            '<table class="timepicker-hours">' +
                                '<tr><td class="timepicker-hour">00</td><td class="timepicker-hour">06</td><td class="timepicker-hour">12</td><td class="timepicker-hour">18</td></tr>' +
                                '<tr><td class="timepicker-hour">01</td><td class="timepicker-hour">07</td><td class="timepicker-hour">13</td><td class="timepicker-hour">19</td></tr>' +
                                '<tr><td class="timepicker-hour">02</td><td class="timepicker-hour">08</td><td class="timepicker-hour">14</td><td class="timepicker-hour">20</td></tr>' +
                                '<tr><td class="timepicker-hour">03</td><td class="timepicker-hour">09</td><td class="timepicker-hour">15</td><td class="timepicker-hour">21</td></tr>' +
                                '<tr><td class="timepicker-hour">04</td><td class="timepicker-hour">10</td><td class="timepicker-hour">16</td><td class="timepicker-hour">22</td></tr>' +
                                '<tr><td class="timepicker-hour">05</td><td class="timepicker-hour">11</td><td class="timepicker-hour">17</td><td class="timepicker-hour">23</td></tr>' +
                            '</table>' +
                        '</td>' +
                        '<td class="timepicker-divider">:</td>' +
                        '<td>' +
                            '<table class="timepicker-minutes">' +
                                '<tr><td class="timepicker-minute">00</td><td class="timepicker-minute">30</td></tr>' +
                                '<tr><td class="timepicker-minute">05</td><td class="timepicker-minute">35</td></tr>' +
                                '<tr><td class="timepicker-minute">10</td><td class="timepicker-minute">40</td></tr>' +
                                '<tr><td class="timepicker-minute">15</td><td class="timepicker-minute">45</td></tr>' +
                                '<tr><td class="timepicker-minute">20</td><td class="timepicker-minute">50</td></tr>' +
                                '<tr><td class="timepicker-minute">25</td><td class="timepicker-minute">55</td></tr>' +
                            '</table>' +
                        '</td>' +
                    '</tr>' +
                '</table>' +
                '<div class="timepicker-hdivider"></div>' +
                '<table class="timepicker-buttons">' +
                    '<tr>' +
                        '<td class="timepicker-now">' +
                            'Now' +
                        '</td>' +
                    '</tr>' +
                    '<tr>' +
                        '<td class="timepicker-clear">' +
                            'Clear' +
                        '</td>' +
                    '</tr>' +
                '</table>' +
            '</div>' +
        '</div>';

}(window.jQuery));