﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.ChangePasswordModel = function () {

    var self = this;

    self.NewPassword1 = ko.observable('');
    self.NewPassword2 = ko.observable('');


    self.HasError = ko.observable(false);
    self.ErrorMessage = ko.observable('');
    self.IsSaving = ko.observable(false);


    self.Save = function () {

        var user = {
            PasswordHash: null      // will be set below
        };

     
        if (CommonCore.IsNullOrEmpty(self.NewPassword1())) {
            self.HasError(true);
            self.ErrorMessage('New Password is required.');
            return;
        }

        if (self.NewPassword1() != self.NewPassword2()) {
            self.HasError(true);
            self.ErrorMessage('New Passwords do not match.');
            return;
        }
            
        
        user.PasswordHash = self.NewPassword1();
    


        self.IsSaving(true);

        CommonCore.PostData('/admin/changepassword/save', ko.toJSON(user), function (data) {

            // success
            self.IsSaving(false);

            var o = $.parseJSON(data);

            if (o.ResultCode === 'OK') {
                // we wil be prompted to enter our new password immedietly since Basic Auth will not be updated... so blank the document
                $(document.body).html('');
                window.location = '/admin';
            } else {
                self.HasError(true);
                self.ErrorMessage(o.Description + ' - ' + o.Details);
            }

        }, function (message) {

            self.IsSaving(false);

            //error
            self.HasError(true);
            self.ErrorMessage('Error saving  - ' + message);

        });
    }
    

};