﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.DatabaseEditModel = function (databaseJson) {
    var self = this;

    self.RowUID = databaseJson.RowUID;
    self.Name = ko.observable(databaseJson.Name);
    self.DatabaseServer = ko.observable(databaseJson.DatabaseServer);
    self.DatabaseName = ko.observable(databaseJson.DatabaseName);
    self.IsCurrentVersion = ko.observable(databaseJson.IsCurrentVersion);
    self.CanConnect = ko.observable(databaseJson.CanConnect);
    self.IsPrimary = databaseJson.IsPrimary;

    self.AgentMode = ko.observable(databaseJson.AgentMode);

    self.AgentModeList = ko.observableArray([
        { Key: 'AllEnabled', Value: 'All Enabled' },
        { Key: 'AllDisabled', Value: 'All Disabled' },
        { Key: 'OnlyInstanceMove', Value: 'Only Instance Move'},
        { Key: 'OnlyProvisioning', Value: 'Only Provisioning' }
    ]);

    self.HasError = ko.observable(false);
    self.ErrorMessage = ko.observable('');
    self.IsSaving = ko.observable(false);

    self.IsLoading = ko.observable(true);

    if (databaseJson.IsNew) {
        self.Title = ko.observable('Attach Database');
    } else {
        self.Title = ko.observable('Edit Database');
    }

    self.IsLoading = ko.observable(false);

    self.Save = function () {

        var database = {
            RowUID: self.RowUID,
            Name: self.Name(),
            DatabaseServer: self.DatabaseServer(),
            DatabaseName: self.DatabaseName(),
            AgentMode: self.AgentMode()
        };

        if (!self.IsPrimary) {

            if (CommonCore.IsNullOrEmpty(database.Name)) {
                self.HasError(true);
                self.ErrorMessage('Name is required.');
                return;
            }

            if (CommonCore.IsNullOrEmpty(database.DatabaseServer)) {
                self.HasError(true);
                self.ErrorMessage('Database Server is required.');
                return;
            }

            if (CommonCore.IsNullOrEmpty(database.DatabaseName)) {
                self.HasError(true);
                self.ErrorMessage('Database Name is required.');
                return;
            }

        }

        self.IsSaving(true);

        CommonCore.PostData('/admin/databases/save', ko.toJSON(database), function (data) {

            // success
            self.IsSaving(false);

            var o = $.parseJSON(data);

            if (o.ResultCode === 'OK') {
                location.href = "/admin/databases";
            } else {
                self.HasError(true);
                self.ErrorMessage(o.Description + ' - ' + o.Details);
            }

        }, function (message) {

            self.IsSaving(false);

            //error
            self.HasError(true);
            self.ErrorMessage('Error saving  - ' + message);

        });
    }

};