﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.DatabaseListModel = function (databasesJson) {
    var self = this;

    self.IsLoading = ko.observable(true);

    self.DatabaseJson = ko.mapping.fromJS(databasesJson);

    self.CurrentTab = ko.observable('None');

    self.ClickHeader = function(columnName) {
        if (self.SortBy() == columnName) {
            self.SortBy(columnName + " desc");
        } else {
            self.SortBy(columnName);
        }
    };

    self.SortBy = ko.observable('Name');

    self.SortedDatabaseJson = function() {
        return self.DatabaseJson().sort(CommonCore.SortByProperty(self.SortBy() + ', Name'));
    };

    self.Groups = CommonCore.koComputedDeferred(function() {

        var arr = [];

        switch (self.CurrentTab()) {

            case 'None':
                arr.push({ Caption: null, Rows: self.SortedDatabaseJson(), FlexDataRowCount: 0, InstanceCount: 0 });
                break;

            case 'Server':
                for (var i = 0; i < self.SortedDatabaseJson().length; i++) {
                    var row = self.SortedDatabaseJson()[i];
                    var curGroup = CommonCore.ArrayFindByPropertyValue(arr, 'Caption', row.DatabaseServer());
                    if (curGroup == null) {
                        curGroup = { Caption: row.DatabaseServer(), Rows: [], FlexDataRowCount: 0, InstanceCount: 0 };
                        arr.push(curGroup);
                    }
                    curGroup.Rows.push(row);
                    curGroup.FlexDataRowCount = curGroup.FlexDataRowCount + row.FlexDataRowCount();
                    curGroup.InstanceCount = curGroup.InstanceCount + row.InstanceCount();
                }
                break;

            case 'Type':
                for (var i = 0; i < self.SortedDatabaseJson().length; i++) {
                    var row = self.SortedDatabaseJson()[i];

                    var types = row.InstanceTypes();
                    var typeArr = CommonCore.StringSplitTrim(types, ',', true);
                    var typeArr2 = [];
                    for (var j = 0; j < typeArr.length; j++) {
                        if (typeArr[j] != 'DELETE' && typeArr[j] != 'CANCELLED') {
                            typeArr2.push(typeArr[j]);
                        }
                    }
                    types = CommonCore.DefaultIfNullOrEmpty(CommonCore.ArrayJoin(', ', typeArr2), '');

                    var curGroup = CommonCore.ArrayFindByPropertyValue(arr, 'Caption', types);
                    if (curGroup == null) {
                        curGroup = { Caption: types, Rows: [], FlexDataRowCount: 0, InstanceCount: 0 };
                        arr.push(curGroup);
                    }
                    curGroup.Rows.push(row);
                    curGroup.FlexDataRowCount = curGroup.FlexDataRowCount + row.FlexDataRowCount();
                    curGroup.InstanceCount = curGroup.InstanceCount + row.InstanceCount();
                }
                break;

            case 'Packages':
                for (var i = 0; i < self.SortedDatabaseJson().length; i++) {
                    var row = self.SortedDatabaseJson()[i];
                    var curGroup = CommonCore.ArrayFindByPropertyValue(arr, 'Caption', row.Packages());
                    if (curGroup == null) {
                        curGroup = { Caption: row.Packages(), Rows: [], FlexDataRowCount: 0, InstanceCount: 0 };
                        arr.push(curGroup);
                    }
                    curGroup.Rows.push(row);
                    curGroup.FlexDataRowCount = curGroup.FlexDataRowCount + row.FlexDataRowCount();
                    curGroup.InstanceCount = curGroup.InstanceCount + row.InstanceCount();
                }
                break;

            default:
                throw new Error('Unknown Tab type: ' + self.CurrentTab());
        } 

        var arr2 = arr.sort(CommonCore.SortByProperty('Caption'));

        return arr2;

    });

    for (var i = 0, row; row = self.DatabaseJson()[i]; i++) {
        row.ChangeTracker = CommonCore.koChangeTracker(row);
        row.IsNew = ko.observable(false);
        row.IsDeleted = ko.observable(false);
        if (row.AgentMode() == 'AllEnabled') {
            row.AgentModeCaption = 'All Enabled';
            row.AgentModeColor = '';
        } else if (row.AgentMode() == 'AllDisabled') {
            row.AgentModeCaption = 'All Disabled';
            row.AgentModeColor = 'color-danger';
        } else if (row.AgentMode() == 'OnlyInstanceMove') {
            row.AgentModeCaption = 'Only Instance Move';
            row.AgentModeColor = 'color-danger';
        } else if (row.AgentMode() == 'OnlyProvisioning') {
            row.AgentModeCaption = 'Only Provisioning';
            row.AgentModeColor = 'color-danger';
        }
    }

    self.UpdateColumns = ko.observable(true);

    self.ShowDeleteColumn = ko.computed(function () {
        // to update if row is deleted
        self.UpdateColumns();
        var rowCount = 0;

        for (var j = 0; j < self.DatabaseJson().length; j++) {
            if (self.DatabaseJson()[j].IsDeleted() == false) {
                rowCount++;
            }
        }
        return rowCount > 1;
    });

    self.ShowUpgradeColumn = ko.computed(function () {
        // to update if row is deleted
        self.UpdateColumns();

        for (var j = 0; j < self.DatabaseJson().length; j++) {
            if (self.DatabaseJson()[j].IsDeleted() == false && !self.DatabaseJson()[j].IsCurrentVersion()) {
                return true;
            }
        }
        return false;
    });

    self.ShowErrorColumn = CommonCore.koComputedDeferred(function () {
        // to update if row is deleted
        self.UpdateColumns();

        for (var k = 0; k < self.DatabaseJson().length; k++) {
            if (self.DatabaseJson()[k].IsDeleted() == false && !CommonCore.IsNullOrEmpty(self.DatabaseJson()[k].SchemaUpgradeLastError())) {
                return true;
            }
        }
        return false;
    });

    self.DeleteDatabase = function (row) {

        var database = ko.mapping.toJS(row);


        CommonCore.DownloadData('databases/delete/' + database.RowUID, function (data) {

            var o = $.parseJSON(data);

            if (o.ResultCode === 'OK') {
                row.IsDeleted(true);
                self.UpdateColumns(!self.UpdateColumns());
            } else {
                alert(o.Description + ' - ' + o.Details);
            }

        }, function (message) {
            
            // error
            alert('Error deleting  - ' + message);

        });
    };

    self.UpgradeDatabase = function(row) {
        var database = ko.mapping.toJS(row);

        CommonCore.DownloadData('databases/upgrade/' + database.RowUID, function (data) {

            var o = $.parseJSON(data);

            if (o.ResultCode === 'OK') {
                row.SchemaUpgradeStatus('Queued');
                row.SchemaUpgradeLastError('');
            } else {
                alert('Error queueing database for upgrade - ' + o.Description + ' - ' + o.Details);
            }
        }, function (message) {
            // error
            alert('Error queueing database for upgrade - ' + message);

        });
    };


    self.SetAllAgentMode = function (mode) {
        if (confirm('Are you sure you wish to set all database agent modes to [' + mode +']?')) {
            CommonCore.DownloadData('databases/setallagentmode/' + mode,
                function(data) {
                    var o = $.parseJSON(data);
                    if (o.ResultCode === 'OK') {
                        window.location.reload(true);
                    } else {
                        alert('Error setting all agent mode - ' + o.Description + ' - ' + o.Details);
                    }
                },
                function(message) {
                    // error
                    alert('Error setting all agent mode - ' + message);

                });
        }
    };

    self.IsLoading(false);

};