﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.InstanceEditModel = function (instanceJson) {
    var self = this;

    self.IsLoading = ko.observable(true);

    self.InstanceUID = instanceJson.InstanceUID;
    self.SiteContentDatabaseUID = instanceJson.SiteContentDatabaseUID;

    self.InstanceName = ko.observable(instanceJson.Instance.InstanceName);
    self.InstanceType = ko.observable(instanceJson.Instance.InstanceType);

    self.CustomerName = ko.observable(instanceJson.Instance.CustomerName);
    self.CustomerAlsoKnownAs = ko.observable(instanceJson.Instance.CustomerAlsoKnownAs);
    self.CustomerHubSpotID = ko.observable(instanceJson.Instance.CustomerHubSpotID);
    self.SiteCaption = ko.observable(instanceJson.Instance.SiteCaption);
    self.ThemeName = ko.observable(instanceJson.Instance.ThemeName);
    self.SiteSuspensionMode = ko.observable(instanceJson.Instance.SiteSuspensionMode);
    self.TrialEndDate = ko.observable(CommonCore.FormatData(instanceJson.Instance.TrialEndDate, 'YMD'));
    self.EmailEnabled = ko.observable(instanceJson.Instance.EmailEnabled);

    self.ApiThrottleDailyCallLimit = ko.observable(instanceJson.Instance.ApiThrottleDailyCallLimit);
    self.ApiThrottleBypassUntilDtm = ko.observable(!CommonCore.IsNullOrEmpty(instanceJson.Instance.ApiThrottleBypassUntilDtm) ? CommonCore.CleanDate(instanceJson.Instance.ApiThrottleBypassUntilDtm) : '');

    self.IsOnline = ko.observable(0);

    //  note: the modal displays a checkbox and it is natural for it to be checked if the instance is online so we will use IsOnline in the modal
    if (instanceJson.Instance.IsOffline == 0) {
        self.IsOnline(1);
    } else {
        self.IsOnline(0);
    }

    self.EnhancedTestingMode = ko.observable(instanceJson.Instance.EnhancedTestingMode);

    self.PasswordMinimumCharacters = ko.observable(instanceJson.Instance.PasswordMinimumCharacters);
    self.PasswordRequireNumber = ko.observable(instanceJson.Instance.PasswordRequireNumber);
    self.PasswordRequireSpecialCharacter = ko.observable(instanceJson.Instance.PasswordRequireSpecialCharacter);
    self.PasswordRequireUpper = ko.observable(instanceJson.Instance.PasswordRequireUpper);
    self.PasswordExpirationDays = ko.observable(instanceJson.Instance.PasswordExpirationDays);
    self.PasswordExpirationWarningDays = ko.observable(instanceJson.Instance.PasswordExpirationWarningDays);
    self.UniquePasswordBeforeReuse = ko.observable(instanceJson.Instance.UniquePasswordBeforeReuse);

    self.ProvisioningPackageFileName = ko.observable(instanceJson.Instance.ProvisioningPackageFileName);
    self.ProvisioningPackageParameters = ko.observable(instanceJson.Instance.ProvisioningPackageParameters);
    self.ProvisioningMajorDeleteInstanceData = ko.observable(instanceJson.Instance.ProvisioningMajorDeleteInstanceData);
    self.ProvisioningMajorAutoDeployEnabled = ko.observable(instanceJson.Instance.ProvisioningMajorAutoDeployEnabled);
    

    self.IsADFSEnabled = instanceJson.IsADFSEnabled;
    self.IsADFSAccountCreationEnabled = instanceJson.IsADFSAccountCreationEnabled;
    self.IsSAMLEnabled = instanceJson.IsSAMLEnabled;
    self.IsSAMLAccountCreationEnabled = instanceJson.IsSAMLAccountCreationEnabled;
    

    self.SiteSuspensionModeList = ko.observableArray([
        { Key: null, Value: '' },
        { Key: 'UnpaidOwners', Value: 'Unpaid - Notify Site Owner Role Only' },
        { Key: 'UnpaidAllAdmins', Value: 'Unpaid - Notify All Administrator Roles' },
        { Key: 'UnpaidEnded', Value: 'Unpaid - Block Entire Site' },
        { Key: 'SiteEnded', Value: 'Cancelled - Block Entire Site' },
        { Key: 'TrialEnded', Value: 'Trial Ended - Block Entire Site' }
    ]);

    self.SelectedTab = ko.observable('General');


    self.SiteUrlMapping = ko.observableArray();

    self.AdfsEndpoint = ko.observable(instanceJson.Instance.AdfsEndpoint);
    self.AdfsImportTemplateUserUID = ko.observable(instanceJson.Instance.AdfsImportTemplateUserUID);
    self.AdfsImportFollowUpEmail = ko.observable(instanceJson.Instance.AdfsImportFollowUpEmail);
    self.AdfsWelcomeMessage = ko.observable(instanceJson.Instance.AdfsWelcomeMessage);

    self.SamlEndpoint = ko.observable(instanceJson.Instance.SamlEndpoint);
    self.SamlImportTemplateUserUID = ko.observable(instanceJson.Instance.SamlImportTemplateUserUID);
    self.SamlImportFollowUpEmail = ko.observable(instanceJson.Instance.SamlImportFollowUpEmail);
    self.SamlWelcomeMessage = ko.observable(instanceJson.Instance.SamlWelcomeMessage);
    self.SamlButtonCaption = ko.observable(instanceJson.Instance.SamlButtonCaption);
    self.SamlCertificate = ko.observable(instanceJson.Instance.SamlCertificate);
    self.SamlResponseUsernameXPath = ko.observable(instanceJson.Instance.SamlResponseUsernameXPath);
    self.SamlResponseEmailAddressXPath = ko.observable(instanceJson.Instance.SamlResponseEmailAddressXPath);
    self.SamlResponseFullNameXpath = ko.observable(instanceJson.Instance.SamlResponseFullNameXpath);
    self.SamlResponseFirstNameXPath = ko.observable(instanceJson.Instance.SamlResponseFirstNameXPath);
    self.SamlResponseLastNameXPath = ko.observable(instanceJson.Instance.SamlResponseLastNameXPath);

    self.UserTemplateAccounts = ko.observableArray(instanceJson.UserTemplateAccounts);

    for (var i = 0; i < instanceJson.SiteUrlMappings.length; i++) {
        self.SiteUrlMapping.push(new AdminCore.UrlMappingModel(instanceJson.SiteUrlMappings[i]));
    }
    
    self.ErrorMessages = ko.observableArray();

    self.Themes = ko.observableArray(AdminCore.Shared.ThemeList);

    self.Schemes = ko.observableArray(['https','http']);
    
    self.Title = instanceJson.Instance.InstanceName;


    self.ToggleEmailEnabledCheckbox = function () {
        if (self.EmailEnabled() == 0) {
            self.EmailEnabled(1);
        } else {
            self.EmailEnabled(0);
        }
    };

    self.ToggleIsOnlineCheckbox = function () {
       if (self.IsOnline() == 0 || self.IsOnline() == null) {
           self.IsOnline(1);
       } else {
           self.IsOnline(0);
       }
    };

    self.ToggleEnhancedTestingMode = function () {
        if (self.EnhancedTestingMode() == 0 || self.EnhancedTestingMode() == null) {
            self.EnhancedTestingMode(1);
        } else {
            self.EnhancedTestingMode(0);
        }
    };


    self.IsOffline = CommonCore.koComputedDeferred(function () {
        //  this computed is used by the save method so the property name matches during assignmen
        if (self.IsOnline() == 0) {
            return 1;
        } else {
            return 0;
        }
    });

    self.MappingFiltered = ko.computed(function () {
        return ko.utils.arrayFilter(self.SiteUrlMapping(), function (mapping) {
            return mapping.IsDeleted() === false;
        });
    });


    self.AddMapping = function () {
        var newMapping = {
            RowUID: CommonCore.GenerateUID(),
            InstanceUID: self.InstanceUID,
            Scheme: '',
            HostHeader: '',
            HostPort: 0,
            IsPrimary: 0,
            ExceptionDetailsEnabled: 0,
            IsDeleted: false,
            IsMaintenance: 0,
            IsDeprecated: 0
        };

        var mappingModel = new AdminCore.UrlMappingModel(newMapping);
        self.SiteUrlMapping.push(mappingModel);
    };

    self.DeleteMapping = function (rowUID) {
        for (var i = 0; i < self.SiteUrlMapping().length; i++) {
            if (self.SiteUrlMapping()[i].RowUID === rowUID) {
                self.SiteUrlMapping()[i].Port(0);
                self.SiteUrlMapping()[i].IsDeleted(true);
            }
        }
    };
   

    self.Save = function () {
       
        var instance = {

            SiteContentDatabaseUID: self.SiteContentDatabaseUID,
            InstanceUID: self.InstanceUID,

            InstanceName: self.InstanceName(),
            InstanceType: self.InstanceType(),
            CustomerName: self.CustomerName(),
            CustomerAlsoKnownAs: self.CustomerAlsoKnownAs(),
            CustomerHubSpotID: CommonCore.TrimAndToNull(self.CustomerHubSpotID()),
            SiteCaption: self.SiteCaption(),
            ThemeName: self.ThemeName(),    
            SiteSuspensionMode: CommonCore.IsNullOrEmpty(self.SiteSuspensionMode()) ? null : self.SiteSuspensionMode(),
            TrialEndDate: self.InstanceType() == 'TRIAL' ? self.TrialEndDate() : null,
            EmailEnabled: self.EmailEnabled(),
            IsOffline: self.IsOffline(),
            EnhancedTestingMode: self.EnhancedTestingMode(),
            ApiThrottleDailyCallLimit: self.ApiThrottleDailyCallLimit(),
            ApiThrottleBypassUntilDtm: CommonCore.IsNullOrEmpty(self.ApiThrottleBypassUntilDtm()) ? null : moment(self.ApiThrottleBypassUntilDtm()).format('YYYY-MM-DD'),
            PasswordMinimumCharacters: self.PasswordMinimumCharacters(),
            PasswordRequireNumber: self.PasswordRequireNumber(),
            PasswordRequireSpecialCharacter: self.PasswordRequireSpecialCharacter(),
            PasswordRequireUpper: self.PasswordRequireUpper(),
            PasswordExpirationDays: self.PasswordExpirationDays(),
            PasswordExpirationWarningDays: self.PasswordExpirationWarningDays(),
            UniquePasswordBeforeReuse: self.UniquePasswordBeforeReuse(),
            ProvisioningPackageFileName: self.ProvisioningPackageFileName(),
            ProvisioningPackageParameters: self.ProvisioningPackageParameters(),
            ProvisioningMajorDeleteInstanceData: self.ProvisioningMajorDeleteInstanceData(),
            ProvisioningMajorAutoDeployEnabled: self.ProvisioningMajorAutoDeployEnabled(),

            AdfsEndpoint: self.AdfsEndpoint(),
            AdfsImportTemplateUserUID: self.AdfsImportTemplateUserUID(),
            AdfsImportFollowUpEmail: self.AdfsImportFollowUpEmail(),
            AdfsWelcomeMessage: self.AdfsWelcomeMessage(),
            SamlEndpoint: self.SamlEndpoint(),
            SamlButtonCaption: self.SamlButtonCaption(),
            SamlCertificate: self.SamlCertificate(),
            SamlImportTemplateUserUID: self.SamlImportTemplateUserUID(),
            SamlImportFollowUpEmail: self.SamlImportFollowUpEmail(),
            SamlWelcomeMessage: self.SamlWelcomeMessage(),
            SamlResponseUsernameXPath: self.SamlResponseUsernameXPath(),
            SamlResponseEmailAddressXPath: self.SamlResponseEmailAddressXPath(),
            SamlResponseFullNameXpath: self.SamlResponseFullNameXpath(),
            SamlResponseFirstNameXPath: self.SamlResponseFirstNameXPath(),
            SamlResponseLastNameXPath: self.SamlResponseLastNameXPath(),

            SiteUrlMappings: self.SiteUrlMapping()
            
        };

        var post = ko.mapping.toJS(instance);

        self.ErrorMessages.removeAll();

        if (CommonCore.IsNullOrEmpty(post.InstanceName)) {
            self.ErrorMessages.push('Instance Name is required.');
        } else if (!CommonCore.IsNullOrEmpty(post.InstanceName) && post.InstanceName.length > 50) {
            self.ErrorMessages.push('Instance Name has to be 50 characters or less.');
        } else if (!CommonCore.IsAlphaNumericOnly(post.InstanceName)) {
            self.ErrorMessages.push('Instance Name contains invalid characters.');
        }

        if (CommonCore.IsNullOrEmpty(post.InstanceType)) {
            self.ErrorMessages.push('Instance Type is required.');
        }

        if (post.InstanceType != CommonCore.ReplaceAll(post.InstanceType, ' ', '')) {
            self.ErrorMessages.push('Instance Type cannot contain spaces.');
        }

        if (CommonCore.IsNullOrEmpty(post.CustomerName)) {
            self.ErrorMessages.push('Customer Name is required.');
        }

        if (CommonCore.IsNullOrEmpty(post.SiteCaption)) {
            self.ErrorMessages.push('Site Caption is required.');
        }

        if (!CommonCore.IsInt32(post.ApiThrottleDailyCallLimit)) {
            self.ErrorMessages.push('API Throttle Daily Call Limit must be a numeric value.');
        }

        if (!CommonCore.IsNullOrEmpty(post.ApiThrottleBypassUntilDtm) && !CommonCore.IsDate(post.ApiThrottleBypassUntilDtm)) {
            self.ErrorMessages.push('Bypass API Throttle Until must be empty or a valid date.');
        } else if (moment(post.ApiThrottleBypassUntilDtm).diff(moment(),'days') > 90) {
            self.ErrorMessages.push('Bypass API Throttle Until may not be more than 90 days in the future.   If a customer need more time for onboarding, they must request it in 90 day increments.');
        }

        if (!CommonCore.IsInt32(post.PasswordMinimumCharacters)) {
            self.ErrorMessages.push('Minimum Password Length must be a numeric value.');
        } else {
            if (CommonCore.ParseInt32(post.PasswordMinimumCharacters) < 8) {
                self.ErrorMessages.push('Minimum Password Length must be at least 8.');
            }
        }

        if (!CommonCore.IsInt32(post.PasswordRequireNumber)) {
            self.ErrorMessages.push('Minimum Numeric Characters must be a numeric value.');
        }

        if (!CommonCore.IsInt32(post.PasswordRequireSpecialCharacter)) {
            self.ErrorMessages.push('Minimum Special Characters must be a numeric value.');
        }

        if (!CommonCore.IsInt32(post.PasswordRequireUpper)) {
            self.ErrorMessages.push('Minimum Uppercase Characters must be a numeric value.');
        }

        if (!CommonCore.IsInt32(post.PasswordExpirationDays)) {
            self.ErrorMessages.push('Password Expiration Days must be a numeric value.');
        }

        if (!CommonCore.IsInt32(post.PasswordExpirationWarningDays)) {
            self.ErrorMessages.push('Password Expiration Warning Days must be a numeric value.');
        }

        if (!CommonCore.IsInt32(post.UniquePasswordBeforeReuse)) {
            self.ErrorMessages.push('Unique Passwords Before Reuse must be a numeric value.');
        } 


        if (!CommonCore.IsNullOrEmpty(post.InstanceType) && post.InstanceType.length > 10) {
            self.ErrorMessages.push('Instance Type has to be 10 characters or less.');
        }

        if (!CommonCore.IsNullOrEmpty(post.CustomerName) && post.CustomerName.length > 50) {
            self.ErrorMessages.push('Customer Name has to be 50 characters or less.');
        }

        if (!CommonCore.IsNullOrEmpty(post.SiteCaption) && post.SiteCaption.length > 50) {
            self.ErrorMessages.push('Site Caption has to be 50 characters or less.');
        }
       

        if (self.IsADFSEnabled) {
            if (!CommonCore.IsNullOrEmpty(post.AdfsEndpoint) && post.AdfsEndpoint.length > 2000) {
                self.ErrorMessages.push('ADFS SSO URL has to be 2000 characters or less.');
            }
        }

        if (self.IsADFSAccountCreationEnabled) {
            if (!CommonCore.IsNullOrEmpty(post.AdfsImportFollowUpEmail) && post.AdfsImportFollowUpEmail.length > 200) {
                self.ErrorMessages.push('ADFS Auto-Import Administrator Email Address has to be 100 characters or less.');
            }
        }


        if (self.IsSAMLEnabled) {
            if (!CommonCore.IsNullOrEmpty(post.SamlEndpoint) && post.SamlEndpoint.length > 2000) {
                self.ErrorMessages.push('SAML SSO URL has to be 2000 characters or less.');
            }
            if (!CommonCore.IsNullOrEmpty(post.SamlButtonCaption) && post.SamlButtonCaption.length > 200) {
                self.ErrorMessages.push('SAML SSO Button Caption has to be 200 characters or less.');
            }
        }

        if (self.IsSAMLAccountCreationEnabled) {            
            if (!CommonCore.IsNullOrEmpty(post.SamlImportFollowUpEmail) && post.SamlImportFollowUpEmail.length > 200) {
                self.ErrorMessages.push('SAML Auto-Import Administrator Email Address has to be 100 characters or less.');
            }
        }



        if (!CommonCore.IsNullOrEmpty(post.ProvisioningPackageFileName) && post.ProvisioningPackageFileName.length > 50) {
            self.ErrorMessages.push('Package File Name must be 50 characters or less.');
        }

        if (!CommonCore.IsNullOrEmpty(post.ProvisioningMajorDeleteInstanceData) && post.ProvisioningMajorDeleteInstanceData == 1) {
            if (post.InstanceType == 'PROD' || post.InstanceType == 'SANDBOX' || post.InstanceType == 'TRIAL') {
                self.ErrorMessages.push('Major Version Delete Instance Data may not be enabled for PROD, SANDBOX, or TRIAL Instance Types.');
            }            
        }

        if (!CommonCore.IsNullOrEmpty(post.ProvisioningMajorAutoDeployEnabled) && post.ProvisioningMajorAutoDeployEnabled == 1) {
            if (post.InstanceType == 'PROD' || post.InstanceType == 'SANDBOX' || post.InstanceType == 'TRIAL') {
                self.ErrorMessages.push('Major Version Auto Deploy may not be enabled for PROD, SANDBOX, or TRIAL Instance Types.');
            }
        }

        if (post.SiteSuspensionMode == "SiteEnded" && post.InstanceType != "CANCELLED") {
            if (!confirm('Are you sure that you wish to set Site Suspension Mode to "Cancelled" without also setting its Instance Type to "CANCELLED"?')) {
                return;
            }        
        }

        if (post.SiteSuspensionMode != "SiteEnded" && post.InstanceType == "CANCELLED") {
            if (!confirm('Are you sure that you wish to set Site Suspension Mode away from "Cancelled" without also setting its Instance Type to something other than "CANCELLED"?')) {
                return;
            }
        }

        if (post.SiteSuspensionMode == "TrialEnded" && post.InstanceType != "DELETE") {
            if (!confirm('Are you sure that you wish to set Site Suspension Mode to "Trial Ended" without also setting its Instance Type to "DELETE"?')) {
                return;
            }
        }


        var primaryCnt = 0;
        for (var i = 0; i < post.SiteUrlMappings.length; i++) {

            post.SiteUrlMappings[i].SiteContentDatabaseUID = self.SiteContentDatabaseUID;

            if (!post.SiteUrlMappings[i].IsDeleted) {

                if (CommonCore.IsNullOrEmpty(post.SiteUrlMappings[i].HostHeader)) {
                    self.ErrorMessages.push('Host Header cannot be empty.');
                }

                if (post.SiteUrlMappings[i].HostHeader != CommonCore.ReplaceAll(post.SiteUrlMappings[i].HostHeader, ' ', '')) {
                    self.ErrorMessages.push('Host Header cannot contain spaces.');
                }

                if (post.SiteUrlMappings[i].HostHeader != post.SiteUrlMappings[i].HostHeader.toLowerCase()) {
                    self.ErrorMessages.push('Host Header must be lowercase.'); //  EXIST
                }

                if (post.SiteUrlMappings[i].HostHeader.length > 50) {
                    self.ErrorMessages.push('Host Header contains too many characters');
                }

                if (CommonCore.IsNullOrEmpty(post.SiteUrlMappings[i].HostPort)) {
                    self.ErrorMessages.push('Port cannot be empty.');
                }

                if (isNaN(post.SiteUrlMappings[i].HostPort)) {
                    self.ErrorMessages.push('Port must be * or a valid number.');
                }

                if (post.SiteUrlMappings[i].IsPrimary == 1 && post.SiteUrlMappings[i].IsDeprecated == 1) {
                    self.ErrorMessages.push('Primary and Deprecated may not be set on the same URL.');
                }


                primaryCnt += post.SiteUrlMappings[i].IsPrimary;
            }
        }

        if (post.SiteUrlMappings.length > 0 && primaryCnt != 1) {
            self.ErrorMessages.push('Only one Primary URL Mapping is allowed.');
        }

        if (self.ErrorMessages().length > 0) {
            return;
        }

        ShowAdminLoadingModal();

        CommonCore.PostData('/admin/instances/save', ko.toJSON(post), function (data) {

            var o = $.parseJSON(data);

            if (o.ResultCode === 'OK') {
                location.href = "/admin/instances/view/" + self.InstanceUID + "/" + self.SiteContentDatabaseUID;
            }
            else if (o.ResultCode === 'Validation') {

                HideAdminLoadingModal();

                var messages = o.Messages;
                CommonCore.forEach(messages, function (message) {
                    self.ErrorMessages.push(message.Message);
                });
            } else {

                HideAdminLoadingModal();

                self.ErrorMessages.push(o.Description + ' - ' + o.Details);
            }
            
        }, function (message) {

            HideAdminLoadingModal();

            //error
            self.ErrorMessages.push('Error saving  - ' + message);
        });
    };

    
    setTimeout(function() {
        self.IsLoading(false);
    },0);

};

AdminCore.UrlMappingModel = function (mapping) {
    var self = this;

    self.RowUID = mapping.RowUID;
    self.InstanceUID = mapping.InstanceUID;
    self.Scheme = mapping.Scheme;
    self.HostHeader = mapping.HostHeader;
    self.Port = ko.observable(mapping.HostPort == 0 ? '*' : mapping.HostPort);
    self.ExceptionDetailsEnabled = ko.observable(mapping.ExceptionDetailsEnabled);
    self.IsPrimary = ko.observable(mapping.IsPrimary);
    self.HostPort = CommonCore.koComputedDeferred(function () {
        if (self.Port() === '*') {
            return 0;
        }

        return self.Port();
    });
    self.SiteContentDatabaseUID = mapping.SiteContentDatabaseUID;
    self.IsDeleted = ko.observable(mapping.IsDeleted);
    self.IsMaintenance = ko.observable(mapping.IsMaintenance);
    self.IsDeprecated = ko.observable(mapping.IsDeprecated);
};

