﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.InstanceKeyEditModel = function (instanceJson) {
    var self = this;

    self.IsLoading = ko.observable(true);

    self.InstanceUID = instanceJson.InstanceUID;
    self.SiteContentDatabaseUID = instanceJson.SiteContentDatabaseUID;

    
    self.ApiKey = ko.observable(instanceJson.ApiKey);
    self.ApiROKey = ko.observable(instanceJson.ApiROKey);
    self.ApiSystemKey = ko.observable(instanceJson.ApiSystemKey);
    self.ApiSystemROKey = ko.observable(instanceJson.ApiSystemROKey);    

    self.ErrorMessages = ko.observableArray();

    self.Title = instanceJson.InstanceName;

    self.Save = function () {
       
        var instance = {
            InstanceUID: self.InstanceUID,
            SiteContentDatabaseUID: self.SiteContentDatabaseUID,

            ApiKey: self.ApiKey(),
            ApiROKey: self.ApiROKey(),
            ApiSystemKey: self.ApiSystemKey(),
            ApiSystemROKey: self.ApiSystemROKey()

        };

        var post = ko.mapping.toJS(instance);

        self.ErrorMessages.removeAll();

        
        if (CommonCore.IsNullOrEmpty(post.ApiKey)) {
            self.ErrorMessages.push('Public (JSON) Full Access API Key is required.');
        }
        if (!CommonCore.IsNullOrEmpty(post.ApiKey) && post.ApiKey.length > 500) {
            self.ErrorMessages.push('Public (JSON) Full Access API Key has to be 500 characters or less.');
        }

        if (CommonCore.IsNullOrEmpty(post.ApiROKey)) {
            self.ErrorMessages.push('Public (JSON) Read Only API Key is required.');
        }
        if (!CommonCore.IsNullOrEmpty(post.ApiROKey) && post.ApiROKey.length > 500) {
            self.ErrorMessages.push('Public (JSON) Read Only API Key has to be 500 characters or less.');
        }

        if (CommonCore.IsNullOrEmpty(post.ApiSystemKey)) {
            self.ErrorMessages.push('System (SDK) Full Access API Key is required.');
        }
        if (!CommonCore.IsNullOrEmpty(post.ApiSystemKey) && post.ApiSystemKey.length > 500) {
            self.ErrorMessages.push('System (SDK) Full Access API Key has to be 500 characters or less.');
        }

        if (CommonCore.IsNullOrEmpty(post.ApiSystemROKey)) {
            self.ErrorMessages.push('System (SDK) Read Only API Key is required.');
        }
        if (!CommonCore.IsNullOrEmpty(post.ApiSystemROKey) && post.ApiSystemROKey.length > 500) {
            self.ErrorMessages.push('System (SDK) Read Only API Key has to be 500 characters or less.');
        }


        if (self.ErrorMessages().length > 0) {
            return;
        }

        ShowAdminLoadingModal();

        CommonCore.PostData('/admin/instances/keysave', ko.toJSON(post), function (data) {

            var o = $.parseJSON(data);

            if (o.ResultCode === 'OK') {
                location.href = "/admin/instances/view/" + self.InstanceUID + "/" + self.SiteContentDatabaseUID;
            }
            else if (o.ResultCode === 'Validation') {

                HideAdminLoadingModal();

                var messages = o.Messages;
                CommonCore.forEach(messages, function (message) {
                    self.ErrorMessages.push(message.Message);
                });

            } else {

                HideAdminLoadingModal();

                self.ErrorMessages.push(o.Description + ' - ' + o.Details);
            }
            
        }, function (message) {

            HideAdminLoadingModal();

            //error
            self.ErrorMessages.push('Error saving  - ' + message);
        });
    };

    
    setTimeout(function() {
        self.IsLoading(false);
    },0);

};


