"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.InstanceMoveModel = function (instanceMoveData) {
    var self = this;

    self.IsLoading = ko.observable(true);

    self.HasError = ko.observable(null);
    self.ErrorMessage = ko.observable(null);

    self.Databases = ko.observableArray(instanceMoveData.Databases);
    self.Instances = ko.observableArray(instanceMoveData.Instances);

    self.DefaultInstance = instanceMoveData.DefaultInstance;    //  this is the default copy from instance to use

    self.FromInstance = ko.observable(null);
    self.FromDatabase = ko.observable(null);

    self.FromInstanceType = CommonCore.koComputedDeferred(function () {
        return self.FromInstance() == null ? null : self.FromInstance().InstanceType;
    });

    self.FromInstanceName = CommonCore.koComputedDeferred(function () {
        return self.FromInstance() == null ? null : self.FromInstance().InstanceName;
    });

    self.FromDatabaseName = CommonCore.koComputedDeferred(function () {
        return self.FromInstance() == null ? null : self.FromInstance().DatabaseName;
    });

    self.FromDatabaseUID = CommonCore.koComputedDeferred(function () {
        return self.FromInstance() == null ? null : self.FromInstance().SiteContentDatabaseUID;
    });

    self.ToDatabase = ko.observable(null);

    self.ToDatabaseName = CommonCore.koComputedDeferred(function () {
        return self.ToDatabase() == null ? null : self.ToDatabase().InstanceName;
    });

    self.FilteredToDatabaes = CommonCore.koComputedDeferred(function () {
        var results = [];

        var databases = self.Databases();

        for (var i = 0; i < databases.length; i++) {
            if (databases[i].RowUID != self.FromDatabaseUID()) {
                results.push(databases[i]);
            }
        }

        return results;
    });

    self.Init = function () {

        if (self.DefaultInstance != null) {
            var instances = self.Instances();
            for (var i = 0; i < instances.length; i++) {
                if (instances[i].InstanceUID == self.DefaultInstance.InstanceUID && instances[i].SiteContentDatabaseUID == self.DefaultInstance.SiteContentDatabaseUID) {
                    self.FromInstance(instances[i]);
                    break;
                }
            }
        }

        self.IsLoading(false);

    }();

    self.Save_Click = function () {
        self.IsLoading(true);

        self.HasError(false);
        self.ErrorMessage(null);

        var postData = {
            FromInstanceUID: self.FromInstance().InstanceUID,
            FromContentDatabaseUID: self.FromInstance().SiteContentDatabaseUID,
            ToInstanceUID: self.FromInstance().InstanceUID, //  todo: [future] we re-use the FromInstanceUID value, but we might let them specify an a new GUID in the UX in a later enhancement
            ToContentDatabaseUID: self.ToDatabase().RowUID
        };

        CommonCore.PostData('/admin/instancemove/add/save', ko.toJSON(postData), function (data) {
            //  success
            var o = CommonCore.ParseJson(data);

            if (o.ResultCode == 'OK') {
                //  move was scheduled so lets go back to the instances page which now displays the instance that is being moved
                window.location = '/admin/instancemove';

            } else {
                //  server returned an error
                self.HasError(true);
                self.ErrorMessage(o.Description);
                self.IsLoading(false);
            }

        }, function (error) {
            self.HasError(true);
            self.ErrorMessage(error);
            self.IsLoading(false);

        }, 60 * 1000);
    };

};