"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.MarketingRequestModel = function (instanceJson, isNew) {
    var self = this;

    self.IsLoading = ko.observable(true);

    self.IsNew = ko.observable(isNew);

    if (isNew == 1) {
        self.Title = ko.observable('Add Marketing Update');
    } else {
        self.Title = ko.observable('Edit Marketing Update');
    }

    self.ErrorMessages = ko.observableArray([]);

    self.MarketingUpdateUID = ko.observable(instanceJson.MarketingUpdateUID);
    self.EmailSubject = ko.observable(instanceJson.EmailSubject);
    self.EmailBody = ko.observable(instanceJson.EmailBody);
    self.IsPublished = ko.observable(instanceJson.IsPublished);

    self.OriginalData = instanceJson.EmailSubject + instanceJson.EmailBody;

    self.DataValidate = function () {
        var errors = [];

        if (CommonCore.IsNullOrEmpty(self.EmailSubject())) {
            errors.push('Subject is required.');
        }

        if (CommonCore.IsNullOrEmpty(self.EmailBody())) {
            errors.push('Message Body is required.');
        }

        return errors;
    };

    self.SaveDraft_Click = function () {
        self.ErrorMessages(self.DataValidate());

        if (self.ErrorMessages().length == 0) {

            var postData = {
                MarketingUpdateUID: self.MarketingUpdateUID(),
                EmailSubject: self.EmailSubject(),
                EmailBody: self.EmailBody(),
                IsNew: self.IsNew() == 1,
                IsPublished: 0
            };

            self.IsLoading(true);

            CommonCore.PostData('/admin/marketing/save', ko.toJSON(postData), function (data) {
                // success
                var o = $.parseJSON(data);

                if (o.ResultCode == 'OK') {
                    // success
                    window.location = '/admin/marketing';

                } else {
                    //  server error
                    self.ErrorMessages.push(o.Description + ' - ' + o.Details);
                    self.IsLoading(false);
                }
            }, function (message) {
                //  client error
                self.ErrorMessages.push(message);
                self.IsLoading(false);
            });
        }
    };

    self.SendPreview_click = function () {
        self.ErrorMessages(self.DataValidate());

        if (self.ErrorMessages().length == 0) {

            var email = prompt("Enter an email address to send a preview of this marketing update to:");

            if (email != null) {

                //  lets try a very simple email check for an @ symbol and a period.  anything more complicated can be resolved by the server
                if (email.indexOf('@') == -1 || email.indexOf('.') == -1) {
                    self.ErrorMessages.push('Preview email address is invalid.');
                    return;
                }

                var postData = {
                    MarketingUpdateUID: self.MarketingUpdateUID(),
                    TestEmailTo: email,
                    EmailSubject: self.EmailSubject(),
                    EmailBody: self.EmailBody(),
                    IsNew: self.IsNew() == 1,
                    IsPublished: 0
                };

                self.IsLoading(true);

                CommonCore.PostData('/admin/marketing/test', ko.toJSON(postData), function (data) {
                    // success
                    var o = $.parseJSON(data);

                    if (o.ResultCode == 'OK') {
                        //  success
                        window.location = '/admin/marketing';
                    } else {
                        //  server error
                        self.ErrorMessages.push(o.Description + ' - ' + o.Details);
                        self.IsLoading(false);

                    }
                }, function (message) {
                    //  client error
                    self.ErrorMessages.push(message);
                    self.IsLoading(false);

                });
            }
        }
    };

    self.Publish_click = function () {
        self.ErrorMessages(self.DataValidate());

        if (self.ErrorMessages().length == 0) {

            var answer = confirm("Do you want to publish and send out this marketing update to customers?");

            if (answer == true) {

                self.IsLoading(true);

                var postData = {
                    MarketingUpdateUID: self.MarketingUpdateUID(),
                    EmailSubject: self.EmailSubject(),
                    EmailBody: self.EmailBody(),
                    IsNew: self.IsNew() == 1,
                    IsPublished: 1
                };

                CommonCore.PostData('/admin/marketing/save', ko.toJSON(postData), function (data) {
                    // success
                    var o = $.parseJSON(data);

                    if (o.ResultCode == 'OK') {
                        //  success
                        window.location = '/admin/marketing';

                    } else {
                        //  server error
                        self.ErrorMessages.push(o.Description + ' - ' + o.Details);
                        self.IsLoading(false);

                    }
                }, function (message) {
                    //  client error
                    self.ErrorMessages.push(message);
                    self.IsLoading(false);

                });
            }
        }
    };

    self.Delete_Click = function () {
        self.ErrorMessages([]);

        self.IsLoading(true);

        CommonCore.DownloadData('/admin/marketing/delete/' + self.MarketingUpdateUID(), function (data) {
            // success            
            var o = $.parseJSON(data);

            if (o.ResultCode == 'OK') {
                //  success
                window.location = '/admin/marketing';

            } else {
                //  server error
                self.ErrorMessages.push(o.Description + ' - ' + o.Details);
                self.IsLoading(false);

            }
        }, function (message) {
            //  client error
            self.ErrorMessages.push(message);
            self.IsLoading(false);

        });
    };

    self.Cancel_Click = function () {
        if((self.OriginalData == self.EmailSubject() + self.EmailBody()) || confirm("Are you sure you want to discard your changes?")) { 
            window.location = '/admin/marketing';
        }
    };

    self.IsLoading(false);
};