﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.ProvisioningListModel = function (instances, packageVersion) {
    var self = this;

    
    self.Instances = ko.observableArray(instances);
    self.PackageVersion = ko.observable(packageVersion);

    self.IsLoading = ko.observable(false);


    self.CurrentTab = ko.observable('Type');

    self.DistinctTypeList = CommonCore.koComputedDeferred(function () {

        var arr = [];
        for (var i = 0; i < self.Instances().length; i++) {

            var instance = self.Instances()[i];

            var type = "";
            if (self.CurrentTab() == "Type") {
                type = instance.InstanceType;
            } else if (self.CurrentTab() == 'Version') {
                type = instance.ConfigVersion;
            } else {
                type = instance.ContentDatabaseName;
            }


            var typeObj = _.find(arr, function (val) { return val.Title == type; });

            if (typeObj === undefined) {
                var sortOrder = type;
                switch (type) {
                case 'BUILD': sortOrder = '0000'; break;
                case 'DEV': sortOrder = '0001'; break;
                case 'PROD': sortOrder = '0002'; break;
                case 'SANDBOX': sortOrder = '0003'; break;
                case 'DEMO': sortOrder = '0004'; break;
                case 'TRIAL': sortOrder = '0005'; break;
                case 'DELETE': sortOrder = 'ZZZZ'; break;
                }
                typeObj = { Title: type, SortOrder: sortOrder, Instances: [] };
                arr.push(typeObj);
            }

            typeObj.Instances.push(instance);

        }


        return arr.sort(CommonCore.SortByProperty('SortOrder'));

    });





    self.UpdatePackageVersion = function() {

        self.IsLoading(true);

        self.PackageVersion(CommonCore.TrimPreserveNull(self.PackageVersion()));        // head off a VERY common copy-paste issue

        setTimeout(function() {

            CommonCore.DownloadData('/admin/provisioning/updateversion?version=' + self.PackageVersion(), function(data) {
                var o = CommonCore.ParseJson(data);
                if (o.ResultCode != "OK") {
                    alert('Error updating version.');
                }
                self.IsLoading(false);
            }, function(err) {
                // error
                alert(err);
                self.IsLoading(false);
            });

        }, 500);       // this is a UX hack... otherwise it does not look like the update actually happened
    };




    self.DeployFarmMajor = function () {
        if (confirm('Are you sure you wish to apply a MAJOR version upgrade to ALL SITES?')) {
            self.DeployUpgrade('major', '', '');
        }
    };

    self.DeployFarmPatch = function () {
        if (confirm('Are you sure you wish to apply a PATCH version upgrade to ALL SITES?')) {
            self.DeployUpgrade('patch', '', '');
        }
    };

    self.DeployInstanceMajor = function (instance) {
        if (confirm('Are you sure you wish to apply a MAJOR version upgrade to ' + instance.InstanceName + '?')) {
            self.DeployUpgrade('major', instance.SiteContentDatabaseUID, instance.InstanceUID);
        }
    };

    self.DeployInstancePatch = function (instance) {
        if (confirm('Are you sure you wish to apply a PATCH version upgrade to ' + instance.InstanceName + '?')) {
            self.DeployUpgrade('patch', instance.SiteContentDatabaseUID, instance.InstanceUID);
        }
    };


    self.DeployUpgrade = function (type, contentDatabaseUID, instanceUID) {

        self.IsLoading(true);

        setTimeout(function() {

            CommonCore.DownloadData('/admin/provisioning/deployupgrade?type=' + type + '&contentDatabaseUID=' + contentDatabaseUID + '&instanceUID=' + instanceUID, function (data) {
                var o = CommonCore.ParseJson(data);
                if (o.ResultCode != "OK") {
                    alert('Error deploying ' + type + ' upgrade.  WARNING: Some jobs may have been scheduled.');
                }
                self.IsLoading(false);
            }, function(err) {
                // error
                alert('Error deploying ' + type + ' upgrade.  WARNING: Some jobs may have been scheduled.   ' + err);
                self.IsLoading(false);
            });

        }, 500); // this is a UX hack... otherwise it does not look like the update actually happened

    };


    self.ExecuteQaInstanceArchive = function() {

        var lastVersion = prompt("What version should the current sites be archived as? (ex: 18.03)");

        if (lastVersion.length != 5) {
            if (!confirm('Are you sure you input the version number correctly?  \r\nMinor version numbers should have leading zeros (ex: 18.03).  \r\nVersion Input: ' + lastVersion)) {
                return;
            }
        }

        self.IsLoading(true);

        setTimeout(function () {

            CommonCore.DownloadData('/admin/provisioning/executeqaarchive?lastversion=' + lastVersion, function (data) {
                var o = CommonCore.ParseJson(data);
                if (o.ResultCode != "OK") {
                    alert('Error archiving instances.  WARNING: Some instances may have been archived.');
                }
                self.IsLoading(false);
            }, function (err) {
                // error
                alert('Error archiving instances.  WARNING: Some instances may have been archived.');
                self.IsLoading(false);
            });

        }, 500); // this is a UX hack... otherwise it does not look like the update actually happened


    };

};
