﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.PulseModel = function () {

    var self = this;


    self.IsLoadingLive = ko.observable(true);
    self.IsLoadingHistory = ko.observable(true);

    self.OnlineActiveFilter = ko.observable('Both');
    self.RecentNavigationsVisible = ko.observable(false);
    self.OnlineClientsVisible = ko.observable(false);
    

    self.PingRows = ko.observableArray([]);
    self.RecentRows = ko.observableArray([]);

    self.UserHistory24HourRows = ko.observableArray([]);
    self.UserHistory24HourDS = new KendoLiveDataSource(self.UserHistory24HourRows, ['PingDtm', 'ActiveUserCount', 'OnlineUserCount']);

    self.PageLoad14DayRows = ko.observableArray([]);
    self.PageLoad14DayDS = new KendoLiveDataSource(self.PageLoad14DayRows, ['RequestDtm', 'FastCount', 'MediumCount', 'SlowCount']);

    self.Request14DayRows = ko.observableArray([]);
    self.Request14DayDS = new KendoLiveDataSource(self.Request14DayRows, ['RequestDtm', 'RequestCount7', 'ApiRequestCount7', 'SyncRequestCount7', 'SlowRequestCount7', 'RequestCount14', 'ApiRequestCount14', 'SyncRequestCount14', 'SlowRequestCount14']);

    self.UserHistory14DayRows = ko.observableArray([]);
    self.UserHistory14DayDS = new KendoLiveDataSource(self.UserHistory14DayRows, ['PingDtm', 'ActiveUserCount7', 'OnlineUserCount7', 'ActiveUserCount14', 'OnlineUserCount14', 'ActiveUserCount21', 'OnlineUserCount21', 'ActiveUserCount28', 'OnlineUserCount28']);

    self.TimeZoneRows = ko.observableArray([]);
    self.TimeZoneDS = new KendoLiveDataSource(self.TimeZoneRows, ['TimeZone', 'ActiveUserCount', 'OnlineWithoutActiveCount']);

    self.DeviceTypeRows = ko.observableArray([]);
    self.DeviceTypeDS = new KendoLiveDataSource(self.DeviceTypeRows, ['DeviceType', 'ActiveUserCount', 'OnlineWithoutActiveCount']);

    self.ByInstanceRows = ko.observableArray([]);
    self.ByInstanceDS = new KendoLiveDataSource(self.ByInstanceRows, ['InstanceName', 'ActiveUserCount', 'OnlineWithoutActiveCount']);

    self.UserPerDayRows = ko.observableArray([]);
    self.UserPerDayDS = new KendoLiveDataSource(self.UserPerDayRows, ['DayDate', 'UniqueUserCount', 'NativeUserOnlineCount', 'NativeUserOfflineCount', 'NativeUserSyncCount', 'NativeSyncBatchCount']);

    self.DurationPerDayRows = ko.observableArray([]);
    self.DurationPerDayDS = new KendoLiveDataSource(self.DurationPerDayRows, ['DayDate', 'MaxDuration', 'AvgDuration']);

    self.SecurityEvent14DayRows = ko.observableArray([]);
    self.SecurityEvent14DayDS = new KendoLiveDataSource(self.SecurityEvent14DayRows, ['EventDtm', 'EventCount7', 'EventCount14', 'AlertCount7', 'AlertCount14']);

    self.SecurityEvent60DayRows = ko.observableArray([]);
    self.SecurityEvent60DayDS = new KendoLiveDataSource(self.SecurityEvent60DayRows, ['DayDate', 'EventCount']);

    self.OnlineCount = ko.observable(0);
    self.ActiveCount = ko.observable(0);


    self.InlineError = ko.observable(null);

    self.SparklineWidth = CommonCore.koComputedDeferred(function() {
        var width = CommonCore.WindowWidth();
        return (width - 220 ) / 3;
    });

    self.IsCondensed = CommonCore.koComputedDeferred(function () {
        return CommonCore.WindowWidth() < 1000;        
    });

    self.ToggleOnlineActive = function (toggle) {
        self.IsLoadingLive(true);
        setTimeout(function() {
            if (toggle == self.OnlineActiveFilter()) {
                self.OnlineActiveFilter('Both');
            } else {
                self.OnlineActiveFilter(toggle);
            }
            self.IsLoadingLive(false);
        }, 0);
    };

    self.DeviceIcons = function(deviceType, browserName, platformName) {

        var html = '';

        switch (deviceType) {
            case 'Browser':
                switch (browserName) {
                    case 'Chrome':
                        html += '<i class="fa fa-fw fa-chrome"></i> ';
                        break;
                    case 'Firefox':
                        html += '<i class="fa fa-fw fa-firefox"></i> ';
                        break;
                    case 'IE11':
                    case 'IE10':
                        html += '<i class="fa fa-fw fa-internet-explorer"></i> ';
                        break;
                    case 'Edge':
                        html += '<i class="fa fa-fw fa-edge"></i> ';
                        break;
                    case 'Safari':
                        html += '<i class="fa fa-fw fa-safari"></i> ';
                        break;
                    case 'Opera':
                        html += '<i class="fa fa-fw fa-opera"></i> ';
                        break;
                    default:
                        html += '<i class="fa fa-fw fa-globe"></i> ';
                }

                if (CommonCore.Contains(platformName, 'Win')) html += '<i class="fa fa-fw fa-windows"></i> ';
                else if (CommonCore.Contains(platformName, 'OSX')) html += '<i class="fa fa-fw fa-desktop"></i> ';
                else if (CommonCore.Contains(platformName, 'macOS')) html += '<i class="fa fa-fw fa-desktop"></i> ';

                else if (CommonCore.Contains(platformName, 'iOS')) html += '<i class="fa fa-fw fa-apple"></i> ';
                else if (CommonCore.Contains(platformName, 'iPhone')) html += '<i class="fa fa-fw fa-apple"></i> ';
                else if (CommonCore.Contains(platformName, 'iPad')) html += '<i class="fa fa-fw fa-apple"></i> ';

                else if (CommonCore.Contains(platformName, 'Android')) html += '<i class="fa fa-fw fa-android"></i> ';

                else if (CommonCore.Contains(platformName, 'Linux')) html += '<i class="fa fa-fw fa-linux"></i> ';

                break;
            case 'iOS':
                html += '<i class="fa fa-fw fa-apple"></i> ';
                break;
            case 'Android':
                html += '<i class="fa fa-fw fa-android"></i> ';
                break;
            case 'Electron':
                html += '<i class="fa fa-fw fa-laptop"></i> ';
                break;
        }

        return html;


    };

    self.AppVersionCss = function(appVersion) {
        var ver1 = CommonCore.GetMajorMinorVersion(appVersion);
        var ver2 = CommonCore.AdminMajorMinorVersion;
        if (ver1 == ver2) {
            return '';
        } else {
            return 'color-danger';
        }
    };


    self.Startup = function() {

        self.RefreshLive();
        self.RefreshHistory();

        Visibility.every(1000 * 5,      1000 * 60 * 1,  self.RefreshLive);                // 5 seconds / 1 minute
        Visibility.every(1000 * 60 * 5, 1000 * 60 * 15, self.RefreshHistory);             // 5 minutes / 15 minutes

    };

    self.RefreshLive = function() {

        CommonCore.DownloadData("/admin/pulse/live", function (data) {

            // success
            var o = CommonCore.ParseJson(data);
            if (o.ResultCode === 'OK') {

                self.OnlineCount(o.OnlineCount);
                self.ActiveCount(o.ActiveCount);
                self.DeviceTypeRows(o.DeviceTypeRows);
                self.TimeZoneRows(o.TimeZoneRows);
                self.UserHistory24HourRows(o.UserHistory24HourRows);
                self.PingRows(o.PingRows);
                self.RecentRows(o.RecentRows);                
                self.ByInstanceRows(o.ByInstanceRows);
                
                self.InlineError(null);
                self.IsLoadingLive(false);

            } else {
                self.InlineError(new CommonCore.InlineError(CommonCore.Concatenate('Error downloading pulse data.', o.Description, '<br/>'), o.Details, o.CorrelationID));
            }

        }, function (message) {

            // error
            self.InlineError(new CommonCore.InlineError('Error downloading pulse data.', message, null));

        }, null, null, true);


    };

    self.RefreshHistory = function () {

        CommonCore.DownloadData("/admin/pulse/history", function (data) {

            // success
            var o = CommonCore.ParseJson(data);
            if (o.ResultCode === 'OK') {

                self.UserPerDayRows(o.UserPerDayRows);
                self.DurationPerDayRows(o.DurationPerDayRows);
                self.PageLoad14DayRows(o.PageLoad14DayRows);
                self.UserHistory14DayRows(o.UserHistory14DayRows);
                self.Request14DayRows(o.Request14DayRows);
                self.SecurityEvent14DayRows(o.SecurityEvent14DayRows);
                self.SecurityEvent60DayRows(o.SecurityEvent60DayRows);

                self.InlineError(null);
                self.IsLoadingHistory(false);

            } else {
                self.InlineError(new CommonCore.InlineError(CommonCore.Concatenate('Error downloading pulse data.', o.Description, '<br/>'), o.Details, o.CorrelationID));
            }

        }, function (message) {

            // error
            self.InlineError(new CommonCore.InlineError('Error downloading pulse data.', message, null));

        }, null, null, true);


    };


    self.InstanceFilter = ko.observable(null);

    self.ToggleInstanceFilter = function(val) {
        if (self.InstanceFilter() == val) {
            self.InstanceFilter(null);
        } else {
            self.InstanceFilter(val);
        }
    };

    self.VisiblePingRows = CommonCore.koComputedDeferred(function() {

        if (self.InstanceFilter() == null) {
            return self.PingRows();
        }

        var ret = [];

        for (var i = 0; i < self.PingRows().length; i++) {
            var row = self.PingRows()[i];
            if (row.InstanceName == self.InstanceFilter()) {
                ret.push(row);
            }
        }
        return ret;

    });

    self.VisibleRecentRows = CommonCore.koComputedDeferred(function () {

        if (self.InstanceFilter() == null) {
            return self.RecentRows();
        }

        var ret = [];

        for (var i = 0; i < self.RecentRows().length; i++) {
            var row = self.RecentRows()[i];
            if (row.InstanceName == self.InstanceFilter()) {
                ret.push(row);
            }
        }
        return ret;

    });




    self.UserHistory24HourChart = function() {

        self.UserHistory24HourRows();     // subscribe
     
        var series = [];
        var axis = [];

        axis.push({
            name: 'Primary',
            min: 0,
            majorUnit: (_.max(self.UserHistory24HourDS.Data.OnlineUserCount) < 8 ? 1 : undefined),
            labels: {
                font: "13px 'Open Sans',Arial,Helvetica,sans-serif"
            },
            majorGridLines:  {
                visible: true,
                color: '#f1f1f1'
            }
        });


        if (self.OnlineActiveFilter() == 'Both' || self.OnlineActiveFilter() == 'Online') {

            series.push({
                name: 'OnlineUsers',
                type: 'area',
                line: { style: 'smooth' },
                color: '#cccccc',
                border: { width: 0 },
                data: self.UserHistory24HourDS.Data.OnlineUserCount
            });

        }

        if (self.OnlineActiveFilter() == 'Both' || self.OnlineActiveFilter() == 'Active') {

            series.push({
                name: 'ActiveUsers',
                type: 'area',
                line: { style: 'smooth' },
                color: '#5CB85C',
                border: { width: 0 },
                data: self.UserHistory24HourDS.Data.ActiveUserCount
            });

        }


        return {

            theme: "bootstrap",

            legend: {
                visible: false,
                position: 'bottom',
                labels: {
                    font: "13px 'Open Sans',Arial,Helvetica,sans-serif"
                }
            },

            valueAxis: axis,

            categoryAxis: {

                categories: self.UserHistory24HourDS.Data.PingDtm,

                visible: true,
                //axisCrossingValues: [0, 9999999999999, 9999999999999, 9999999999999, 9999999999999],
                labels: {
                    font: "13px 'Open Sans',Arial,Helvetica,sans-serif",
                    step: 4,
                    template: kendo.template("#= moment.utc(value).local().format('HH') #")
                },
                majorGridLines: {
                    visible: false
                }
                
            },
        
            transitions: false,

            tooltip: {
                padding: 10,
                visible: true,
                format: "{0}",
                template: "#= category # (#= value #)"
            },

            series: series

        };

    };



    self.SecurityEvent7Chart = function () {
        return self.BuildLineChart(
            self.SecurityEvent14DayDS.Data.EventDtm,
            self.SecurityEvent14DayDS.Data.EventCount7,
            self.SecurityEvent14DayDS.Data.EventCount14,
            null,
            null,
            '#fcad37', '#dddddd', '#f3f3f3', '#f3f3f3',
            kendo.template("#= moment.utc(value).local().format('HH:mm') == '12:00' ? moment.utc(value).local().format('ddd') : '' #"),
            0);
    };


    self.UserHistoryOnlineChart = function () {
        return self.BuildLineChart(
            self.UserHistory14DayDS.Data.PingDtm,
            self.UserHistory14DayDS.Data.OnlineUserCount7,
            self.UserHistory14DayDS.Data.OnlineUserCount14,
            self.UserHistory14DayDS.Data.OnlineUserCount21,
            self.UserHistory14DayDS.Data.OnlineUserCount28,
            '#5CB85C', '#dddddd', '#f3f3f3', '#f3f3f3',
            kendo.template("#= moment.utc(value).local().format('HH:mm') == '12:00' ? moment.utc(value).local().format('ddd') : '' #"),
            0);
    };

    self.UserHistoryActiveChart = function () {
        return self.BuildLineChart(
            self.UserHistory14DayDS.Data.PingDtm,
            self.UserHistory14DayDS.Data.ActiveUserCount7,
            self.UserHistory14DayDS.Data.ActiveUserCount14,
            self.UserHistory14DayDS.Data.ActiveUserCount21,
            self.UserHistory14DayDS.Data.ActiveUserCount28,
            '#5CB85C', '#dddddd', '#f3f3f3', '#f3f3f3',
            kendo.template("#= moment.utc(value).local().format('HH:mm') == '12:00' ? moment.utc(value).local().format('ddd') : '' #"),
            0);
    };


    self.RequestChart = function () {
        return self.BuildLineChart(
            self.Request14DayDS.Data.RequestDtm,
            self.Request14DayDS.Data.RequestCount7,
            self.Request14DayDS.Data.RequestCount14,
            null,
            null,
            '#5CB85C', '#BAE2BA', '#E7FFE7', '#E7FFE7',
            kendo.template("#= moment.utc(value).local().format('HH:mm') == '12:00' ? moment.utc(value).local().format('ddd') : '' #"),
            0);
            //'#007ACC', '#ADD8F4', '', '');
    };

    self.RequestApiChart = function () {
        return self.BuildLineChart(
            self.Request14DayDS.Data.RequestDtm,
            self.Request14DayDS.Data.ApiRequestCount7,
            self.Request14DayDS.Data.ApiRequestCount14,
            null,
            null,
            '#5CB85C', '#dddddd', '#f3f3f3', '#f3f3f3',
            kendo.template("#= moment.utc(value).local().format('HH:mm') == '12:00' ? moment.utc(value).local().format('ddd') : '' #"),
            0);
            //'#007ACC', '#ADD8F4', '', '');
    };

    self.RequestSyncChart = function () {
        return self.BuildLineChart(
            self.Request14DayDS.Data.RequestDtm,
            self.Request14DayDS.Data.SyncRequestCount7,
            self.Request14DayDS.Data.SyncRequestCount14,
            null,
            null,
            '#5CB85C', '#dddddd', '#f3f3f3', '#f3f3f3',
            kendo.template("#= moment.utc(value).local().format('HH:mm') == '12:00' ? moment.utc(value).local().format('ddd') : '' #"),
            0);
        //'#007ACC', '#ADD8F4', '', '');
    };

    self.RequestSlowChart = function () {
        return self.BuildLineChart(
            self.Request14DayDS.Data.RequestDtm,
            self.Request14DayDS.Data.SlowRequestCount7,
            self.Request14DayDS.Data.SlowRequestCount14,
            null,
            null,
            '#DA4932', '#dddddd', '#f3f3f3', '#f3f3f3',
            kendo.template("#= moment.utc(value).local().format('HH:mm') == '12:00' ? moment.utc(value).local().format('ddd') : '' #"),
            0);
        //'#007ACC', '#ADD8F4', '', '');
    };

    self.DurationPerDayChart = function () {
        return self.BuildLineChart(
            self.DurationPerDayDS.Data.DayDate,
            self.DurationPerDayDS.Data.MaxDuration,
            self.DurationPerDayDS.Data.AvgDuration,
            null,
            null,
            '#2083D6', '#5DC4EA', '#888888', '#888888',
            kendo.template("#= moment.utc(value).format('ddd') #"),
            100,
            ['Max Duration', 'Avg Duration']);
    };


    self.PageLoad14DayChart = function () {
        return self.BuildLineChart(
          self.PageLoad14DayDS.Data.RequestDtm,
          self.PageLoad14DayDS.Data.SlowCount,
          self.PageLoad14DayDS.Data.MediumCount,          
          self.PageLoad14DayDS.Data.FastCount,
          null,
          '#DA4932', '#888888', '#5CB85C', '#f1f1f1',
          kendo.template("#= moment.utc(value).format('ddd') #"),
          100, 
          ['>7 seconds','3-7 seconds','<3 seconds']);
    };


    self.BuildLineChart = function (categories, data7, data14, data21, data28, color7, color14, color21, color28, categoryTemplate, rotateThreshold, legendCaptions) {

        self.UserHistory14DayRows();     // subscribe

        var series = [];
        var axis = [];

        axis.push({
            name: 'Primary',
            min: 0,
            majorUnit: (_.max(data7) + _.max(data14)) < 8 ? 1 : undefined,
            labels: {
                font: "13px 'Open Sans',Arial,Helvetica,sans-serif"
            },
            majorGridLines: {
                visible: true,
                color: '#f1f1f1'
            }
        });

        if (data28 != null) {
            series.push({
                name: (legendCaptions !== undefined && legendCaptions !== null) ? legendCaptions[3] : 'Data28',
                type: 'line',
                style: 'smooth',
                color: color28,
                border: { width: 0 },
                markers: { visible: false },
                data: data28
            });
        }

        if (data21 != null) {
            series.push({
                name: (legendCaptions !== undefined && legendCaptions !== null) ? legendCaptions[2] : 'Data21',
                type: 'line',
                style: 'smooth',
                color: color21,
                border: { width: 0 },
                markers: { visible: false },
                data: data21
            });
        }

        series.push({
            name: (legendCaptions !== undefined && legendCaptions !== null) ? legendCaptions[1] : 'Data14',
            type: 'line',
            style: 'smooth',
            color: color14,
            border: { width: 0 },
            markers: { visible: false },
            data: data14
        });

        series.push({
            name: (legendCaptions !== undefined && legendCaptions !== null) ? legendCaptions[0] : 'Data7',
            type: 'line',
            style: 'smooth',
            color: color7,
            border: { width: 0 },
            markers: { visible: false },
            data: data7
        });



        return {

            theme: "bootstrap",

            legend: {
                visible: (legendCaptions !== undefined && legendCaptions !== null && legendCaptions.length > 0),
                position: 'bottom',
                labels: {
                    font: "13px 'Open Sans',Arial,Helvetica,sans-serif"
                }
            },

            valueAxis: axis,

            categoryAxis: {

                categories: categories,
                
                visible: true,
                //axisCrossingValues: [0, 9999999999999, 9999999999999, 9999999999999, 9999999999999],
                labels: {
                    font: "13px 'Open Sans',Arial,Helvetica,sans-serif",
                    rotation: ((categories.length * rotateThreshold) < CommonCore.WindowWidth()) ? 0 : 90,
                    step: 1,
                    template: categoryTemplate
                },
                majorGridLines: {
                    visible: false
                }

            },

            transitions: false,

            tooltip: {
                padding: 10,
                visible: true,
                format: "{0}",
                template: "#= moment.utc(category).local().format('YYYY-MM-DD HH:mm') # (#= value #)"
            },

            plotArea: {
                margin: { top: 0, left: 10, right: 10, bottom: 0 }
            },

            series: series

        };

    };


    self.TimeZoneChart = function () {
        self.TimeZoneRows();        // subscribe
        switch (self.OnlineActiveFilter()) {
        case 'Both':
            return self.StackedBarChart(self.TimeZoneDS.Data.ActiveUserCount, self.TimeZoneDS.Data.OnlineWithoutActiveCount, self.TimeZoneDS.Data.TimeZone);
        case 'Active':
            return self.StackedBarChart(self.TimeZoneDS.Data.ActiveUserCount, [], self.TimeZoneDS.Data.TimeZone);
        case 'Online':
            return self.StackedBarChart([], self.TimeZoneDS.Data.OnlineWithoutActiveCount, self.TimeZoneDS.Data.TimeZone);
        }
    };

    self.DeviceTypeChart = function () {
        self.DeviceTypeRows();      // subscribe
        switch (self.OnlineActiveFilter()) {
            case 'Both':
                return self.StackedBarChart(self.DeviceTypeDS.Data.ActiveUserCount, self.DeviceTypeDS.Data.OnlineWithoutActiveCount, self.DeviceTypeDS.Data.DeviceType);
            case 'Active':
                return self.StackedBarChart(self.DeviceTypeDS.Data.ActiveUserCount, [], self.DeviceTypeDS.Data.DeviceType);
            case 'Online':
                return self.StackedBarChart([], self.DeviceTypeDS.Data.OnlineWithoutActiveCount, self.DeviceTypeDS.Data.DeviceType);
        }
    };

    self.ByInstanceChart = function () {
        self.ByInstanceRows();      // subscribe
        switch (self.OnlineActiveFilter()) {
        case 'Both':
            return self.StackedBarChart(self.ByInstanceDS.Data.ActiveUserCount, self.ByInstanceDS.Data.OnlineWithoutActiveCount, self.ByInstanceDS.Data.InstanceName);
        case 'Active':
            return self.StackedBarChart(self.ByInstanceDS.Data.ActiveUserCount, [], self.ByInstanceDS.Data.InstanceName);
        case 'Online':
            return self.StackedBarChart([], self.ByInstanceDS.Data.OnlineWithoutActiveCount, self.ByInstanceDS.Data.InstanceName);
        }
    };

    self.UserPerDayChart = function () {
        return self.StackedBarChart(self.UserPerDayDS.Data.UniqueUserCount, null, self.UserPerDayDS.Data.DayDate, kendo.template("#= moment.utc(value).format('ddd') #"));
    };

    self.NativeUserOnlineChart = function () {
        return self.StackedBarChart(self.UserPerDayDS.Data.NativeUserOnlineCount, null, self.UserPerDayDS.Data.DayDate, kendo.template("#= moment.utc(value).format('ddd') #"));
    };

    self.NativeUserOfflineChart = function () {
        return self.StackedBarChart(self.UserPerDayDS.Data.NativeUserOfflineCount, null, self.UserPerDayDS.Data.DayDate, kendo.template("#= moment.utc(value).format('ddd') #"));
    };

    self.NativeUserSyncChart = function () {
        return self.StackedBarChart(self.UserPerDayDS.Data.NativeUserSyncCount, null, self.UserPerDayDS.Data.DayDate, kendo.template("#= moment.utc(value).format('ddd') #"));
    };

    self.NativeSyncBatchChart = function () {
        return self.StackedBarChart(self.UserPerDayDS.Data.NativeSyncBatchCount, null, self.UserPerDayDS.Data.DayDate, kendo.template("#= moment.utc(value).format('ddd') #"));
    };


    self.SecurityEvent60Chart = function () {
        return self.StackedBarChart(self.SecurityEvent60DayDS.Data.EventCount, null, self.SecurityEvent60DayDS.Data.DayDate, kendo.template("#= moment.utc(value).format('ddd') #"), '#fcad37');
    };


    self.StackedBarChart = function (series1, series2, category, categoryTemplate, color1, color2) {

        var series = [];
        var axis = [];

        var majorUnit;
        if (series2 == null) {
            majorUnit = (_.max(series1)) < 8 ? 1 : undefined;
        } else {
            majorUnit = (_.max(series1) + _.max(series2)) < 8 ? 1 : undefined;
        }

        axis.push({
            name: 'Primary',
            min: 0,            
            labels: {
                font: "13px 'Open Sans',Arial,Helvetica,sans-serif"
            },
            majorUnit: majorUnit,
            majorGridLines: {
                visible: true,                
                color: '#f1f1f1'
            }
        });

        series.push({
            name: 'Series1',
            type: 'column',
            //type: 'area',
            //line: { style: 'smooth' },
            stack: 'Primary',            
            color: CommonCore.DefaultIfNullOrEmpty(color1,'#5CB85C'),
            border: { width: 0 },
            data: series1
        });

        if (series2 != null) {

            series.push({
                name: 'Series2',
                type: 'column',
                //type: 'area',
                //line: { style: 'smooth' },
                stack: 'Primary',
                color: CommonCore.DefaultIfNullOrEmpty(color2,'#cccccc'),
                border: { width: 0 },
                data: series2
            });

        }
        
        return {

            theme: "bootstrap",

            legend: {
                visible: false,
                position: 'bottom',
                labels: {
                    font: "13px 'Open Sans',Arial,Helvetica,sans-serif"
                }
            },

            valueAxis: axis,

            categoryAxis: {

                categories: category,

                visible: true,
                //axisCrossingValues: [0, 9999999999999, 9999999999999, 9999999999999, 9999999999999],
                labels: {
                    rotation: ((category.length * 120) < CommonCore.WindowWidth()) ? 0 : 90,
                    font: "13px 'Open Sans',Arial,Helvetica,sans-serif",
                    template: categoryTemplate
                },
                majorGridLines: {
                    visible: false                    
                }
            },

            transitions: false,

            tooltip: {
                padding: 10,
                visible: true,
                format: "{0}"
            },

            series: series

        };

    };



};


var KendoLiveDataSource = function(obsArray, properties) {

    var self = this;

    self.Array = obsArray;
    self.Properties = properties;

    self.Data = {};

    for (var i = 0; i < self.Properties.length; i++) {
        self.Data[self.Properties[i]] = [];
    }

    self.Refresh = function() {
        for (var j = 0; j < self.Properties.length; j++) {
            var prop = self.Properties[j];
            var arr = self.Data[prop];
            CommonCore.ArrayClear(arr);
            for (var i = 0; i < self.Array().length; i++) {
                arr.push(self.Array()[i][prop]);
            }
        }
    };

    self.SubscribeObsArray = obsArray.subscribe(function() {
        self.Refresh();
    });


    self.Refresh();

};