﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.ReportEmailQueueModel = function (startDate, endDate, emailToKeyword, subjectKeyword) {

    var self = this;

    self.IsLoading = ko.observable(true);

    self.StartDate = ko.observable(moment(startDate));
    self.EndDate = ko.observable(moment(endDate));

    self.EmailToKeyword = ko.observable(emailToKeyword);
    self.SubjectKeyword = ko.observable(subjectKeyword);

    self.Range = CommonCore.koComputedDeferred(function() {
        return self.StartDate().format('YYYY-MM-DD') + ' to ' + self.EndDate().format('YYYY-MM-DD');
    });

    $('input[name="reportrange"]').daterangepicker({
        startDate: self.StartDate(),
        endDate: self.EndDate(),
        format: 'YYYY-MM-DD',
        ranges: {
            'Today': [moment().utc(), moment().utc()],
            'Yesterday': [moment().utc().subtract(1, 'days'), moment().utc().subtract(1, 'days')],
            'Last 7 Days': [moment().utc().subtract(6, 'days'), moment().utc()],
            'Last 30 Days': [moment().utc().subtract(29, 'days'), moment().utc()],
            'This Month': [moment().utc().startOf('month'), moment().utc().endOf('month')],
            'Last Month': [moment().utc().subtract(1, 'month').startOf('month'), moment().utc().subtract(1, 'month').endOf('month')]
        },
        opens: 'left',
        drops: 'down',
        buttonClasses: ['btn', 'btn-sm'],
        applyClass: 'btn-primary',
        cancelClass: 'btn-default',
        separator: ' to ',
        locale: {
            applyLabel: 'Apply',
            cancelLabel: 'Cancel',
            fromLabel: 'From',
            toLabel: 'To',
            customRangeLabel: 'Custom',
            daysOfWeek: ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'],
            monthNames: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
            firstDay: 1
        }
    }, function (start, end) {
        self.StartDate(moment(start, 'YYYY-MM-DD'));
        self.EndDate(moment(end, 'YYYY-MM-DD'));
    });


    self.CancelEmail = function(contentDatabaseUID, instanceUID, rowUID, event) {
        
        if (confirm('Only use this feature for emails that are corrupt.  Are you sure you want to cancel this email?')) {

            CommonCore.DownloadData('/admin/reports/emailqueue/cancel/' + contentDatabaseUID + '/' + instanceUID + '/' + rowUID,
                function(data) {

                    var o = CommonCore.ParseJson(data);
                    if (o.ResultCode === 'OK') {

                        event.target.outerHTML = '';

                    } else {
                        alert(o.Description);
                    }

                },
                function(message) {
                    alert(message);
                });

        }            
    };


    self.Submit = function() {

        window.location = '/admin/reports/emailqueue?start=' + self.StartDate().format('YYYY-MM-DD') + '&end=' + self.EndDate().format('YYYY-MM-DD') + '&emailtokeyword=' + self.EmailToKeyword() + '&subjectkeyword=' + self.SubjectKeyword();
    };

};

