﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.ReportRowCountPerModel = function (tableRecordCount) {
    var self = this;

    self.TableRecordCount = tableRecordCount;
    self.WithLogicalDeletes = ko.observable(false);
    self.ShowRootRows = ko.observable(false);

    // create header row
    self.Title = ko.observable('');
    self.Header = ko.observableArray();
    self.Header.push('Table');
    CommonCore.forEach(self.TableRecordCount, function (obj) {
        self.Header.push(obj.InstanceName);
        self.Title(obj.Title);
    });

    // builds Array Table With Logical Deleted Items
    self.LogicalDelete = ko.observableArray();
    CommonCore.forEach(self.TableRecordCount, function (obj) {
        if (self.LogicalDelete().length === 0) {
            CommonCore.forEach(obj.TableCount, function (t) {
                var row = [];
                row.push(t.Name);
                row.push(t.Count);
                self.LogicalDelete.push(row);
            });
        } else {
            CommonCore.forEach(obj.TableCount, function (t, i) {
                self.LogicalDelete()[i].push(t.Count);
            });
        }
    });

    // builds Array Table Without Logical Deleted Items
    self.NoLogicalDelete = ko.observableArray();
    CommonCore.forEach(self.TableRecordCount, function (obj) {
        if (self.NoLogicalDelete().length === 0) {
            CommonCore.forEach(obj.TableCount, function (t) {
                var row = [];
                row.push(t.Name);
                row.push(t.Count - t.RowStatus);
                self.NoLogicalDelete.push(row);
            });
        } else {
            CommonCore.forEach(obj.TableCount, function (t, i) {
                self.NoLogicalDelete()[i].push(t.Count - t.RowStatus);
            });
        }
    });

    // RootCount With Logical Deleted Items
    self.RootCountWithLogicalDelete = ko.observableArray();
    CommonCore.forEach(self.TableRecordCount, function (obj) {
        if (self.RootCountWithLogicalDelete().length === 0) {
            CommonCore.forEach(obj.TableCount, function (t) {
                var row = [];
                row.push(t.Name);
                row.push(t.RootCount);
                self.RootCountWithLogicalDelete.push(row);
            });
        } else {
            CommonCore.forEach(obj.TableCount, function (t, i) {
                self.RootCountWithLogicalDelete()[i].push(t.Count);
            });
        }
    });

    // RootCount Without Logical Deleted Items
    self.RootCountWithoutLocgicalDelete = ko.observableArray();
    CommonCore.forEach(self.TableRecordCount, function (obj) {
        if (self.RootCountWithoutLocgicalDelete().length === 0) {
            CommonCore.forEach(obj.TableCount, function (t) {
                var row = [];
                row.push(t.Name);
                row.push(t.RootCount - t.RootRowStatus);
                self.RootCountWithoutLocgicalDelete.push(row);
            });
        } else {
            CommonCore.forEach(obj.TableCount, function (t, i) {
                self.RootCountWithoutLocgicalDelete()[i].push(t.RootCount - t.RootRowStatus);
            });
        }
    });

    // Selects sorting column
    self.SortedColumn = ko.observable(0);
    self.sort = function (header) {
        self.SortedColumn(_.indexOf(self.Header(), header));
    };

    self.Body = ko.computed(function () {
        // selects the tables data
        var rows;
        if (self.WithLogicalDeletes() === true) {
            if (self.ShowRootRows() === true) {
                rows = self.RootCountWithLogicalDelete();
            } else {
                rows = self.LogicalDelete();
            }
        } else {
            if (self.ShowRootRows() === true) {
                rows = self.RootCountWithoutLocgicalDelete();
            } else {
                rows = self.NoLogicalDelete();
            }
        }

        // sorts selected data
        if (self.SortedColumn() === 0) {
            return rows.sort(function(a, b) {
                if (a[self.SortedColumn()] < b[self.SortedColumn()]) return -1;
                if (a[self.SortedColumn()] > b[self.SortedColumn()]) return 1;
                return 0;
            });
        } else {
            // sorts on selected column
            return rows.sort(function(a, b) {
                return b[self.SortedColumn()] - a[self.SortedColumn()];
            });
        }

    });
};