"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.ReportSubmitFeedbackQueueModel = function (viewObject) {
    var self = this;

    self.IsLoading = ko.observable(true);

    self.StartDate = ko.observable(moment(viewObject.StartDate));
    self.EndDate = ko.observable(moment(viewObject.EndDate));


    self.FilterRoute = CommonCore.koComputedDeferred(function() {
        return '/admin/reports/submitfeedbackqueue/' + self.StartDate().format("YYYY-MM-DD") + '/' + self.EndDate().format("YYYY-MM-DD");
    });

    self.IsUnsentErrorChecked = ko.observable(false);
    self.ToggleUnsentError = function() {
        self.IsUnsentErrorChecked(!self.IsUnsentErrorChecked());
    };

    self.FormattedRows = CommonCore.koComputedDeferred(function() {
        var unformattedRows = viewObject.Rows;
        var rows = [];
        for(var i = 0; i <unformattedRows.length ;i++) {
            unformattedRows[i].EmailSubject = new CondensedItem("View Subject", unformattedRows[i].EmailSubject);
            unformattedRows[i].EmailBody = new CondensedItem("View Body", unformattedRows[i].EmailBody);
            unformattedRows[i].LastError = new CondensedItem(CommonCore.IsNullOrEmpty(unformattedRows[i].LastError) ? '': "View Error",unformattedRows[i].LastError);
            unformattedRows[i].LastAttemptDtm = moment.utc(unformattedRows[i].LastAttemptDtm, moment.ISO_8601).isValid() ? moment.utc(unformattedRows[i].LastAttemptDtm, moment.ISO_8601).format('YYYY-MM-DD HH:mm:ss'):'';
            unformattedRows[i].SentDtm = !CommonCore.IsNullOrEmpty(unformattedRows[i].SentDtm) ? moment(unformattedRows[i].SentDtm).format('YYYY-MM-DD HH:mm:ss'):'';
            rows.push(unformattedRows[i]);
        }
        return rows;
    });

    self.FilteredRows = CommonCore.koComputedDeferred(function () {
        var rows = self.FormattedRows();

        // Sort By Unsent / Error
        if (self.IsUnsentErrorChecked()) {
            rows = _.where(rows, {SentDtm: ""});
        }

        return rows;
    });

    self.Range = CommonCore.koComputedDeferred(function() {
        return self.StartDate().format('YYYY-MM-DD') + ' to ' + self.EndDate().format('YYYY-MM-DD');
    });

    $('input[name="reportrange"]').daterangepicker({
        startDate: self.StartDate(),
        endDate: self.EndDate(),
        format: 'YYYY-MM-DD',
        ranges: {
            'Today': [moment().utc(), moment().utc()],
            'Yesterday': [moment().utc().subtract(1, 'days'), moment().utc().subtract(1, 'days')],
            'Last 7 Days': [moment().utc().subtract(6, 'days'), moment().utc()],
            'Last 30 Days': [moment().utc().subtract(29, 'days'), moment().utc()],
            'This Month': [moment().utc().startOf('month'), moment().utc().endOf('month')],
            'Last Month': [moment().utc().subtract(1, 'month').startOf('month'), moment().utc().subtract(1, 'month').endOf('month')]
        },
        opens: 'left',
        drops: 'down',
        buttonClasses: ['btn', 'btn-sm'],
        applyClass: 'btn-primary',
        cancelClass: 'btn-default',
        separator: ' to ',
        locale: {
            applyLabel: 'Apply',
            cancelLabel: 'Cancel',
            fromLabel: 'From',
            toLabel: 'To',
            customRangeLabel: 'Custom',
            daysOfWeek: ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'],
            monthNames: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
            firstDay: 1
        }
    }, function (start, end) {
        self.StartDate(moment(start, 'YYYY-MM-DD'));
        self.EndDate(moment(end, 'YYYY-MM-DD'));
    });

    self.IsLoading(false);


    self.CancelEmail = function(row) {
        if (confirm('Only use this feature for submit feedback request that are corrupt.  Are you sure you want to cancel this request?')) {
            window.location = '/admin/reports/submitfeedbackqueue/cancel/' + row.ContentDatabaseUID + '/' + row.InstanceUID + '/' + row.RowUID;
        }
    };

};


function CondensedItem(caption, body) {
    var self = this;
    self.IsHidden = ko.observable(true);
    self.ToggleHidden = function () {
        self.IsHidden(!self.IsHidden());
    };
    self.Caption = caption;
    self.All = body;
};