﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.ScheduledJobListModel = function (viewObject) {

    var self = this;

    self.IsLoading = ko.observable(true);

    self.HasError = ko.observable(false);
    self.ErrorMessage = ko.observable(null);

    self.NewJobsFound = ko.observable(false);
    self.NewJobsFoundCaption = ko.observable('New Jobs Found');



    self.ShortenServerName = function (serverName) {
        if (!CommonCore.IsNullOrEmpty(serverName)) {
            if (serverName.indexOf('-') >= 0) {
                return serverName.split('-').reverse()[0];
            }
        }
        return serverName;
    };

    self.FormatScheduledJobs = CommonCore.koComputedDeferred(function () {

        return self.ScheduledJobs();

    }, function (data) {
        CommonCore.ForEach(data, function (job, i) {
            if (job.ProvisionSuccessDtm != null) {
                data[i].ProvisionSuccessDtm = moment.utc(data[i].ProvisionSuccessDtm).local().format('YYYY-MM-DD HH:mm:ss');
            }
            data[i].CreatedDtm = moment.utc(data[i].CreatedDtm, moment.ISO_8601).local().format('YYYY-MM-DD HH:mm:ss');

            if (CommonCore.IsNullOrEmpty(data[i].InstanceName)) {
                data[i].InstanceName = '';
            }

            data[i].DurationObservable = data[i].DurationObservable || ko.observable(data[i].Duration);
            data[i].MessageCountObservable = data[i].MessageCountObservable || ko.observable(data[i].MessageCount);
            data[i].ProcessingServerObservable = data[i].ProcessingServerObservable || ko.observable(self.ShortenServerName(data[i].ProcessingServer));
            data[i].StepCountObservable = data[i].StepCountObservable || ko.observable(data[i].StepCount);
            data[i].StepCompleteCountObservable = data[i].StepCompleteCountObservable || ko.observable(data[i].StepCompleteCount);
            data[i].ProcessingStatusObservable = data[i].ProcessingStatusObservable || ko.observable(data[i].ProcessingStatus);
            data[i].ProvisionSuccessDtmObservable = data[i].ProvisionSuccessDtmObservable || ko.observable(data[i].ProvisionSuccessDtm);

        });
        self.ScheduledJobs(data);
    });

    var instanceList = viewObject.InstanceList;
    self.InstanceList = ko.observableArray(viewObject.InstanceList);
    self.InstanceList.sort(CommonCore.SortByProperty('InstanceName'));

    self.ScheduledJobs = ko.observableArray();
    self.FormatScheduledJobs(viewObject.ScheduledJobs);

    self.SelectedInstance = ko.observable();
    self.StartDate = ko.observable();
    self.EndDate = ko.observable();

    if (!CommonCore.IsNullOrEmpty(viewObject.StartDate) && !CommonCore.IsNullOrEmpty(viewObject.EndDate)) {
        self.StartDate(moment(viewObject.StartDate));
        self.EndDate(moment(viewObject.EndDate));
    } else {
        self.StartDate(moment().utc().subtract(2, 'days'));
        self.EndDate(moment().utc().add(1,'days'));
    }

    self.SearchText = ko.observable('');
    self.SearchTextDelayed = ko.computed(function() { return self.SearchText(); }).extend({throttle: 500});



    self.ErrorMessage = ko.observable('');

    self.SortedBy = ko.observable('RCreatedDtm');

    if (viewObject.InstanceUID !== null) {
        for (var i = 0; i < self.InstanceList().length; i++) {
            if (self.InstanceList()[i].InstanceUID == viewObject.InstanceUID && self.InstanceList()[i].SiteContentDatabaseUID == viewObject.SiteContentDatabaseUID) {
                self.SelectedInstance(self.InstanceList()[i]);
            }
        }
    }

    self.FilterRoute = CommonCore.koComputedDeferred(function() {
        var route;

        if (self.SelectedInstance() === undefined) {
            route = '/admin/scheduledjobs/' + self.StartDate().format("YYYY-MM-DD") + '/' + self.EndDate().format("YYYY-MM-DD");
        } else {
            route = '/admin/scheduledjobs/' + self.StartDate().format("YYYY-MM-DD") + '/' + self.EndDate().format("YYYY-MM-DD") + '/' + self.SelectedInstance().InstanceUID + '/' + self.SelectedInstance().SiteContentDatabaseUID;
        }

        return route;

    });

    self.Sort = function (column) {
        if (self.SortedBy() !== column || self.SortedBy() === 'R' + column) {
            self.SortedBy(column);
            self.ScheduledJobs.sort(CommonCore.SortByProperty(column + ',CreatedDtm,Ordinal'));
        } else if (column === self.SortedBy()) {
            self.SortedBy('R' + column);
            self.ScheduledJobs.sort(CommonCore.SortByProperty(column + ' desc,CreatedDtm desc,Ordinal desc'));
        }
    };

    self.ReSort = function() {
        var column = self.SortedBy();
        if (column.substr(0, 1) == 'R') {
            column = column.substr(1);
            self.ScheduledJobs.sort(CommonCore.SortByProperty(column + ' desc,CreatedDtm desc,Ordinal desc'));
        } else {
            self.ScheduledJobs.sort(CommonCore.SortByProperty(column + ',CreatedDtm,Ordinal'));
        }
    };

    self.Range = CommonCore.koComputedDeferred(function () {
        var range;
        if (self.StartDate() === undefined || self.EndDate() === undefined) {
            range = moment().utc().subtract(29, 'days').format('YYYY-MM-DD') + ' to ' + moment().utc().format('YYYY-MM-DD');
        } else {
            range = self.StartDate().format('YYYY-MM-DD') + ' to ' + self.EndDate().format('YYYY-MM-DD');
        }
        return range;
    });

    $('input[name="reportrange"]').daterangepicker({
        startDate: self.StartDate(),
        endDate: self.EndDate(),
        format: 'YYYY-MM-DD',
        ranges: {
            'Today': [moment().utc(), moment().utc()],
            'Yesterday': [moment().utc().subtract(1, 'days'), moment().utc().subtract(1, 'days')],
            'Past 7 Days': [moment().utc().subtract(6, 'days'), moment().utc()],
            'Past 30 Days': [moment().utc().subtract(29, 'days'), moment().utc()],
            'Past Month': [moment().utc().subtract(1, 'month'), moment().utc()],
            'Past Year': [moment().utc().subtract(1, 'year'), moment().utc()]
        },
        opens: 'left',
        drops: 'down',
        buttonClasses: ['btn', 'btn-sm'],
        applyClass: 'btn-primary',
        cancelClass: 'btn-default',
        separator: ' to ',
        locale: {
            applyLabel: 'Apply',
            cancelLabel: 'Cancel',
            fromLabel: 'From',
            toLabel: 'To',
            customRangeLabel: 'Custom',
            daysOfWeek: ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'],
            monthNames: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
            firstDay: 1
        }
    }, function (start, end) {
        self.StartDate(moment(start, 'YYYY-MM-DD'));
        self.EndDate(moment(end, 'YYYY-MM-DD'));
    });




    // summary panel
    var initialPendingCnt = 0;
    var initialStart = moment();
    self.SummaryPanel = CommonCore.koComputedDeferred(function () {

        var currentPendingCnt = 0;
        var queuedCnt = 0;
        var underwayCnt = 0;
        var errorCnt = 0;
        var pausedCnt = 0;

        var underwayList = [];
        var errorList = [];
      
        for (var i = 0; i < self.ScheduledJobs().length; i++) {
            var j = self.ScheduledJobs()[i];

            switch (j.ProcessingStatusObservable()) {
                case 'InProgress':
                    underwayCnt++;
                    currentPendingCnt++;
                    underwayList.push(j.InstanceName);                    
                    break;
                case 'Queued':
                case 'QueuedRetry':
                    queuedCnt++;
                    currentPendingCnt++;
                    break;
                case 'Error':
                    errorCnt++;
                    currentPendingCnt++;
                    errorList.push(j.InstanceName);
                    break;
                case 'Paused':
                case 'Postponed':
                    pausedCnt++;
                    currentPendingCnt++;
                    break;
            }
        }

        if (currentPendingCnt > initialPendingCnt) {
            initialPendingCnt = currentPendingCnt;
            initialStart = moment();
        }

        if (currentPendingCnt === 0) {
            initialPendingCnt = 0;
        }

        var completeCnt = (initialPendingCnt - currentPendingCnt);

        var progressSegments = [];
        
        for (var i = 0; i < completeCnt; i++) {
            progressSegments.push({ Color: '#5CB85C' });        // green
        }        
        for (var i = 0; i < errorCnt; i++) {                    
            progressSegments.push({ Color: '#ee5037' });        // red
        }
        for (var i = 0; i < pausedCnt; i++) {                    
            progressSegments.push({ Color: '#EBD686' });       // yellow
        }
        for (var i = 0; i < underwayCnt ; i++) {
            progressSegments.push({ Color: '#4C94FF' });        // blue
        }
        for (var i = 0; i < queuedCnt; i++) {                  
            progressSegments.push({ Color: '#f5f5f5' });        // gray
        }


        return {
            InitialPending: initialPendingCnt,
            CurrentPending: currentPendingCnt,
            Complete: completeCnt,
            Queued: queuedCnt,
            Underway: underwayCnt,
            Error: errorCnt,
            UnderwayList: underwayList,
            ErrorList: errorList,
            ProgressSegments: progressSegments,
            ProgressWidth: (1 / initialPendingCnt) * 100,
            ElapsedTime: moment.utc(moment().diff(initialStart)).format('HH:mm:ss')
    };

    });
    

    self.VisibleScheduledJobs = CommonCore.koComputedDeferred(function() {

        return CommonCore.FilterRowsByKeywords(self.ScheduledJobs(), self.SearchTextDelayed(), ['ProcessingStatus','CommandType','InstanceName','DatabaseName','FileName','ProvisionSuccessDtm','CreatedDtm', 'Duration']);

    });


    self.ErrorCount = CommonCore.koComputedDeferred(function() {

        var cnt = 0;

        for (var i = 0; i < self.ScheduledJobs().length; i++) {
            var j = self.ScheduledJobs()[i];

            if (j.ProcessingStatusObservable() == 'Error') cnt++;

        }

        return cnt;

    });

    self.CancelAllErrors_Click = function() {

        if (confirm('Are you sure that you wish to cancel all errors?')) {
            self.UpdateAllErrors('Cancel');
        }

    };

    self.RetryAllErrors_Click = function () {

        if (confirm('Are you sure that you wish to retry all errors?')) {
            self.UpdateAllErrors('QueuedRetry');
        }

    };

    self.UpdateAllErrors = function(status) {

        for (var i = 0; i < self.ScheduledJobs().length; i++) {

            var j = self.ScheduledJobs()[i];

            if (j.ProcessingStatusObservable() == 'Error') {

                var packageUpdate = {
                    processingStatus: status
                };

                CommonCore.PostData('/admin/scheduledjobs/view/' + j.InstanceUID + '/' + j.SiteContentDatabaseUID  + '/' + j.ProvisioningUID + '/updatequeuestatus', ko.toJSON(packageUpdate), function (data) {

                    var o = jQuery.parseJSON(data);

                    if (o.ResultCode === 'OK') {

                        j.ProcessingStatusObservable(status);

                        self.UpdateAllErrors(status);

                    } else {
                        self.HasError(true);
                        self.ErrorMessage('Update status failed, try again. Error: ' + o.Description);
                    }
                }, function (message) {
                    self.HasError(true);
                    self.ErrorMessage('Update status failed, try again. Error: ' + message);
                });


                break;
            }

        }

    };





    self.IntervalCounter = 0;

    // schedule auto-refresh
    self.IntervalTimerID = setInterval(function() {

        // if we have been polling over an hour, stop polling (b/c it is expensive server-side)
        self.IntervalCounter++;
        if (self.IntervalCounter > (20*60)) {
            clearInterval(self.IntervalTimerID);
            self.NewJobsFoundCaption('Refresh Suspended');
            self.NewJobsFound(true);
            return;
        }
        
        var url = '/admin/scheduledjobs/refresh';
        if (!CommonCore.IsNullOrEmpty(viewObject.StartDate)) {
            url += '/' + viewObject.StartDate + '/' + viewObject.EndDate;
            if (!CommonCore.IsNullOrEmpty(viewObject.InstanceUID)) {
                url += '/' + viewObject.InstanceUID + '/' + viewObject.SiteContentDatabaseUID;
            }
        }

        CommonCore.DownloadData(url, function (data) {

            var o = jQuery.parseJSON(data);

            if (o.ResultCode == 'OK') {

                self.HasError(false);                
                self.ErrorMessage(null);

                for (var i = 0; i < o.Result.ScheduledJobs.length; i++) {

                    var found = false;
                    var job = o.Result.ScheduledJobs[i];

                    for (var j = 0; j < self.ScheduledJobs().length; j++) {

                        var localJob = self.ScheduledJobs()[j];

                        if (localJob.ProvisioningUID == job.ProvisioningUID) {

                            localJob.Duration = job.Duration;
                            localJob.MessageCount = job.MessageCount;
                            localJob.ProcessingServer = self.ShortenServerName(job.ProcessingServer);
                            localJob.StepCount = job.StepCount;
                            localJob.StepCompleteCount = job.StepCompleteCount;
                            localJob.ProcessingStatus = job.ProcessingStatus;
                            localJob.ProvisionSuccessDtm = (job.ProvisionSuccessDtm != null) ? moment.utc(job.ProvisionSuccessDtm).local().format('YYYY-MM-DD HH:mm:ss') : '';

                            localJob.DurationObservable(localJob.Duration);
                            localJob.MessageCountObservable(localJob.MessageCount);
                            localJob.ProcessingServerObservable(localJob.ProcessingServer);
                            localJob.StepCountObservable(localJob.StepCount);
                            localJob.StepCompleteCountObservable(localJob.StepCompleteCount);
                            localJob.ProcessingStatusObservable(localJob.ProcessingStatus);
                            localJob.ProvisionSuccessDtmObservable(localJob.ProvisionSuccessDtm);

                            found = true;

                        }

                    }

                    if (!found) {

                        self.NewJobsFound(true);

                    }

                }

                for (var j = 0; j < self.ScheduledJobs().length; j++) {

                    var localJob = self.ScheduledJobs()[j];

                    var found = false;
                    for (var i = 0; i < o.Result.ScheduledJobs.length; i++) {

                        var job = o.Result.ScheduledJobs[i];
                        if (localJob.ProvisioningUID == job.ProvisioningUID) {
                            found = true;
                        }

                    }
                    if (!found) {
                        localJob.ProcessingStatus = 'Complete';
                        localJob.ProcessingStatusObservable('Complete');        // job was probably a delete-instance job, so just mark it complete
                    }
                }

                self.ReSort();

            } else {

                self.HasError(true);
                self.ErrorMessage('Error retrieving status: ' + o);
            }


        }, function (message) {
            self.HasError(true);
            self.ErrorMessage('Error retrieving status: ' + message);
        });

    }, 3000);


    self.IsLoading(false);
};

