﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.UserEditModel = function (userJson) {
    var self = this;

    self.IsLoading = ko.observable(true);

    self.IsNew = userJson.IsNew;
    self.RowUID = userJson.RowUID;

    self.UserLogin = ko.observable(userJson.UserLogin);
    self.AuthProvider = ko.observable(userJson.AuthProvider);
    self.UserType = ko.observable(userJson.UserType);
    self.EmailAddress = ko.observable(userJson.EmailAddress);

    self.HasError = ko.observable(false);
    self.ErrorMessage = ko.observable('');
    self.IsSaving = ko.observable(false);

    self.AuthProviders = [{ Name: 'ActiveDirectory' }, { Name: 'Internal' }];

    self.UserTypes = [{ Name: 'Read' }, { Name: 'Create' }, { Name: 'Manage' }, { Name: 'DevOps'}, { Name: 'Full' }];

    if (userJson.IsNew) {
        self.Title = ko.observable('Add Site Admin Portal Administrator');
    } else {
        self.Title = ko.observable('Edit Site Admin Portal Administrator');
    }

    self.ReInvite = function () {
        self.Save(true);
    };

    self.Save = function (sendinvite) {

        sendinvite = CommonCore.DefaultIfNullOrUndefined(sendinvite, false);

        var user = {
            RowUID: self.RowUID,
            UserLogin: self.UserLogin(),
            EmailAddress: self.EmailAddress(),
            AuthProvider: self.AuthProvider(),
            UserType: self.UserType(),
            SendInvite: sendinvite,
            IsNew: self.IsNew
        };

        if (CommonCore.IsNullOrEmpty(user.UserLogin)) {
            self.HasError(true);
            self.ErrorMessage('Login is required.');
            return;
        }

        if (CommonCore.IsNullOrEmpty(user.EmailAddress)) {
            self.HasError(true);
            self.ErrorMessage('Email Address is required.');
            return;
        }

        if (CommonCore.IsNullOrEmpty(user.UserType)) {
            self.HasError(true);
            self.ErrorMessage('User Type is required.');
            return;
        }

        if (CommonCore.IsNullOrEmpty(user.AuthProvider)) {
            self.HasError(true);
            self.ErrorMessage('Auth Provider is required.');
            return;
        }


        self.IsSaving(true);

        CommonCore.PostData('/admin/users/save', ko.toJSON(user), function (data) {

            // success
            self.IsSaving(false);

            var o = $.parseJSON(data);

            if (o.ResultCode === 'OK') {
                location.href = "/admin/users";
            } else {
                self.HasError(true);
                self.ErrorMessage(o.Description + (CommonCore.IsNullOrEmpty(o.Details) ? '' : ' - ' + o.Details));
            }

        }, function (message) {

            self.IsSaving(false);

            //error
            self.HasError(true);
            self.ErrorMessage('Error saving  - ' + message);

        });
    };

    self.IsLoading(false);

};