﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AppCore = AppCore || {};

AppCore.SML = AppCore.SML || {};

AppCore.SML.Status = {
    tests: null,
    testID: -1,
    commandID: -1,
    state: "stopped" // enum(running, singlestep, paused, stopped, error)
}

AppCore.SML.QUnit = {
    IsActive: false,
    Assert: null
}

    
/* ----------------------------------------------------------
    * 
    * Tests:
    * - Cycles through one test at a time and reports back when done.
    * 
    ---------------------------------------------------------- */

AppCore.SML.Tests = AppCore.SML.Tests || {};

AppCore.SML.Tests.Start = function (tests) {
    CommonCore.log("AppCore.SML.Tests.Start");
    CommonCore.log("AppCore.SML.SiteName: " + AppCore.UserContext().SiteCurrentUrl.split("/")[2]);
    // Update status
    if (AppCore.SML.GritterID) {
        $.gritter.remove(AppCore.SML.GritterID, { fade: false });
    }
    AppCore.SML.GritterID = CommonCore.ShowGritter('SML Tests - Starting', 'Starting tests...', true, 'warning', 0, null);
    // Prepare script
    AppCore.SML.Status.tests = tests;
    AppCore.SML.Status.testID = -1;
    AppCore.SML.Status.state = "running";
    // Set up environment
    AppCore.IsAutomatedTestRunning(true);
    // Prepare QUnit
    if (AppCore.SML.QUnit.IsActive === true) {
        expect(AppCore.SML.Status.tests.length);
    }
    // Start test
    AppCore.SML.Tests.NextTest();
}
AppCore.SML.Tests.NextTest = function () {
    CommonCore.log("AppCore.SML.Tests.NextTest");
    // Prepare for next test
    AppCore.SML.Status.testID++;
    // Stop if last test was already executed
    if (AppCore.SML.Status.testID >= AppCore.SML.Status.tests.length) {
        AppCore.SML.Tests.Done();
        return;
    }
    CommonCore.log("AppCore.SML.Test.TestName: " + AppCore.SML.Status.tests[AppCore.SML.Status.testID].name);
    // Start test
    AppCore.SML.Test.Start();
}
AppCore.SML.Tests.Done = function () {
    CommonCore.log("AppCore.SML.Tests.Done");
    AppCore.SML.Status.state = 'stopped';
    // Report results of all QUnit tests
    if (AppCore.SML.QUnit.IsActive === true) {
        QUnit.start();
    }
    // Report results to user
    AppCore.IsAutomatedTestRunning(false);
    $.gritter.remove(AppCore.SML.GritterID, { fade: false });
    AppCore.SML.GritterID = CommonCore.ShowGritter('SML Tests - Ready', 'All tests ready...', true, 'warning', 0, null);
}


/* ----------------------------------------------------------
    * 
    * Test:
    * - Runs through all commands of a test.
    * - Reports back when done or when an error occurs.
    * 
    ---------------------------------------------------------- */

AppCore.SML.Test = AppCore.SML.Test || {};

AppCore.SML.Test.Start = function () {
    CommonCore.log("AppCore.SML.Test.Start");
    // Clean up environment
    window.location.hash = '/tester';
    setTimeout(function () {
        // Prepare a clean environment. If a (validation) modal is showing, close it
        if (CommonCore.ModalHelper.IsModalVisible()) {
            if (CommonCore.ModalHelper.Modal[0].id == "UIDiscardChangesModal") {
                CommonCore.ModalHelper.Model.OKButtonClick();
            } else if (CommonCore.ModalHelper.Modal[0].id == "FatalErrorModal") {
                CommonCore.ModalHelper.HideModal();
            } else {
                CommonCore.ModalHelper.ClearAllModals();
            }
        }
        // Prepare script and run first command
        AppCore.SML.Status.commandID = -1;
        AppCore.SML.Test.NextCommand();
    }, 1000);
}
AppCore.SML.Test.NextCommand = function () {
    // Pull the next line that needs to be executed.
    AppCore.SML.Status.commandID++;
    var line = AppCore.SML.Status.tests[AppCore.SML.Status.testID].commands[AppCore.SML.Status.commandID];
    // Stop if last command in test was already executed
    if (typeof line === 'undefined') {
        AppCore.SML.Test.Done();
        return;
    }
    // Halt if test is stopped or paused from the Test Editor
    if (AppCore.SML.Status.tests[0].group == 'Ad Hoc' && (AppCore.SML.Status.state == 'paused' || AppCore.SML.Status.state == 'stopped')) {
        return;
    }
    // Halt on next command if running just one line from Test Editor
    if (AppCore.SML.Status.state == 'singlestep') {
        AppCore.SML.Status.state = 'paused';
    }
    // Log
    CommonCore.log("AppCore.SML.Test.NextCommand (" + (AppCore.SML.Status.commandID + 1) + ") " + line.command);
    // Update status
    $.gritter.remove(AppCore.SML.GritterID, { fade: false });
    AppCore.SML.GritterID = CommonCore.ShowGritter('SML Tests', 'Test: ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].name + ' (' + (AppCore.SML.Status.testID + 1) + ' of ' + AppCore.SML.Status.tests.length + ')<br/>Group: ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].group + '<br/>Line: ' + (AppCore.SML.Status.commandID + 1) + ' of ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].commands.length + '<br/>Command:<ul><li>' + line.command + (line.parameters.length > 0 ? '</li></ul>Parameters:<br/><ul><li>' + line.parameters.join('</li><li>') + '</li></ul>' : ''), true, 'warning', 0, null);
    // Check if command exists;
    if (typeof AppCore.SML.Commands[line.command.toLowerCase()] == 'undefined') {
        AppCore.SML.Test.Error('Command ' + line.command + ' is not implemented.');
        return;
    }
    // Execute command
    try {
        AppCore.SML.Commands[line.command.toLowerCase()].apply(null,
            [
                function () { AppCore.SML.Test.NextCommand(); }, // pass callback
                function (error) { AppCore.SML.Test.Error(error); }, // fail callback
                line.parameters // parameters array
            ].concat(line.parameters)); // individual for named parameters
    }
    catch (e) {
        // If a fatal error happens, report it and then throw it to the regular error handler
        AppCore.SML.Test.Error(e);
        throw e;
        return;
    }
}
AppCore.SML.Test.Done = function () {
    CommonCore.log("AppCore.SML.Test.Done");
    // Report positive status to QUnit
    if (AppCore.SML.QUnit.IsActive === true) {
        AppCore.SML.QUnit.Assert.ok(true, 'Test: ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].name + ' (' + (AppCore.SML.Status.testID + 1) + ' of ' + AppCore.SML.Status.tests.length + '), Group: ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].group);
    }
    // Notify test is done
    AppCore.SML.Tests.NextTest();
}
AppCore.SML.Test.Error = function (error) {
    CommonCore.log("AppCore.SML.Test.Error");
    if (!error) {
        error = "";
    }
    AppCore.SML.Status.state = 'error';
    // Update status
    $.gritter.remove(AppCore.SML.GritterID, { fade: false });
    AppCore.SML.GritterID = CommonCore.ShowGritter('SML Tests - Error', 'Test: ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].name + ' (' + (AppCore.SML.Status.testID + 1) + ' of ' + AppCore.SML.Status.tests.length + ')<br/>Group: ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].group + '<br/>Line: ' + (AppCore.SML.Status.commandID + 1) + ' of ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].commands.length + '<br/>Command:<ul><li>' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].commands[AppCore.SML.Status.commandID].command + '</li></ul>Error:<br/>' + error, true, 'danger', 0, null);
    // Report negative status to QUnit
    if (AppCore.SML.QUnit.IsActive === true) {
        AppCore.SML.QUnit.Assert.ok(false, 'Test: ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].name + ' (' + (AppCore.SML.Status.testID + 1) + ' of ' + AppCore.SML.Status.tests.length + '), Group: ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].group + ', Line: ' + (AppCore.SML.Status.commandID + 1) + ' of ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].commands.length + ', Command: ' + AppCore.SML.Status.tests[AppCore.SML.Status.testID].commands[AppCore.SML.Status.commandID].command + ', Error:' + error);
        // Run next test
        AppCore.SML.Tests.NextTest();
    }
    // Stop tests when not running QUnit
    return;
}





/* ----------------------------------------------------------
    * 
    * Commands
    * 
    ---------------------------------------------------------- */

AppCore.SML.Commands = AppCore.SML.Commands || {};

AppCore.SML.Commands['comment'] = function (pass, fail, parameters) {
    // Ignore
    pass();
}
AppCore.SML.Commands['opendashboard'] = function (pass, fail, parameters, dashboardName) {
    // Navigate to dashboard
    window.location.hash = '/' + dashboardName;
    // Wait for dashboard to be loaded
    AppCore.SML.Helpers.waitUntilDone(
        function () {
            return (
                PageCore.Model.CurrentPage().DashboardMgr.DashboardName.toLowerCase() == dashboardName.toLowerCase() &&
                !PageCore.Model.CurrentPage().DashboardMgr.IsLoading() &&
                PageCore.Model.CurrentPage().DashboardMgr.Panels.every(function (panel) { return !panel.IsLoading() })
            );
        },
        pass,
        function () { fail('Unable to open dashboard ' + dashboardName + '.'); }
    );
}
AppCore.SML.Commands['addnewform'] = function (pass, fail, parameters, entityName) {
    // Navigate to form page
    window.location.hash = '/entity/' + entityName + '/add';
    // Wait for form page to be loaded
    AppCore.SML.Helpers.waitUntilDone(
        function () { return PageCore.Pages.entity.Form().IsLoadComplete() && PageCore.Pages.entity.EntityName == entityName },
        pass,
        function () { fail('Unable to open entity ' + entityName + '.'); }
    );
}
AppCore.SML.Commands['reopensavedform'] = function (pass, fail, parameters) {
    // Similate click on the "Saved" gritter once it shows
    AppCore.SML.Helpers.waitUntilDone(
        function () {
            if (document.getElementById('LastSaveLink'))
            {
                document.getElementById('LastSaveLink').click(); return true;
            } 
        },
        function () {
            // Workaround; the regular waitUntilDone below should be sufficient but there seem to be exceptions every now and then where it results in a false positive, hence adding a default wait period.
            setTimeout(function () {
                // Wait for form page to be loaded
                AppCore.SML.Helpers.waitUntilDone(
                    function () { return PageCore.Pages.entity.Form().IsLoadComplete() && PageCore.Model.CurrentPage().Form().WaitListManager.TotalCounter == 0 && !PageCore.Model.CurrentPage().Form().IsModalSpinner() && !PageCore.Model.CurrentPage().Form().IsTransparentSpinner() },
                    pass,
                    function () { fail('Unable to open last saved form.'); }
                );
            }, 5000);
        },
        function () { fail('Form reopen gritter did not show in a timely manner.'); }
    );
}
AppCore.SML.Commands['clickworkflowbutton'] = function (pass, fail, parameters, buttonCode) {
    var button = PageCore.Model.CurrentPage().Form().SortedVisibleWorkflowButtons().find(function (button) { return button.ButtonCode == buttonCode });
    if (button) {
        // Click button
        PageCore.Model.CurrentPage().Form().WorkflowButtonClick(button);
        // Wait for (optional) spinner to disappear
        setTimeout(function () {
            AppCore.SML.Helpers.waitUntilDone(
                function () { return !PageCore.Pages.entity.Form() || PageCore.Model.CurrentPage().Form().WaitListManager.TotalCounter==0 },
                pass,
                function () { fail('Workflow button click did not process in a timely manner.'); }
            );
        }, 1000);
    } else {
        fail('Button with code ' + buttonCode + ' not found.');
    }
}
AppCore.SML.Commands['clickentitylink'] = function (pass, fail, parameters, fieldName) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field) {
                var hash = field.Url();
                var entityName = field.EntityName();
                // Navigate to form page
                window.location.hash = hash;
                // Wait for form page to be loaded
                AppCore.SML.Helpers.waitUntilDone(
                    function () { return PageCore.Pages.entity.Form().IsLoadComplete() && PageCore.Pages.entity.EntityName == entityName },
                    pass,
                    function () { fail('Unable to open entity ' + entityName + '.'); }
                );
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['setfieldvalue'] = function (pass, fail, parameters, fieldName, valueFormula) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field) {
                PageCore.Model.CurrentPage().Form().WorkflowSetGridFieldValue_Impl(field, field, field.FieldDef.FieldName, valueFormula);
                pass();
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['setfielduomvalue'] = function (pass, fail, parameters, fieldName, valueFormula) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field) {
                var val = PageCore.Model.CurrentPage().Form().ExpressionEngine.EvalValue(PageCore.Model.CurrentPage().Form().RootRow, null, valueFormula);
                field.UomValue(val);
                pass();
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['addgridrow'] = function (pass, fail, parameters, fieldName) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field) {
                field.AddRow(null, true);
                pass();
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['setcheckboxstate'] = function (pass, fail, parameters, fieldName, checkboxUidFormula, valueFormula) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field) {
                var checkboxUidValue = PageCore.Model.CurrentPage().Form().ExpressionEngine.EvalValue(PageCore.Model.CurrentPage().Form().RootRow, null, checkboxUidFormula);
                if (field.FieldDef.FieldType == 'HierarchyPicker') {
                    // MultiHierarchyPicker
                    var checkbox = field.EffectiveFullList().find(function (item) { return item.Row.RowUID == checkboxUidValue; });
                    if (checkbox) {
                        var val = PageCore.Model.CurrentPage().Form().ExpressionEngine.EvalValue(PageCore.Model.CurrentPage().Form().RootRow, null, valueFormula);
                        checkbox.IsChecked(val);
                        pass();
                    } else {
                        fail('Unable to find business entity ' + checkboxUidFormula + ' on hierarchy picker ' + fieldName + '.');
                    }
                } else {
                    // CheckBoxList and potentially other field types
                    var checkbox = field.FilteredOptionsList().find(function (item) { return item.Row.RowUID == checkboxUidValue; });
                    if (checkbox) {
                        checkbox.SetCheckboxValue(valueFormula);
                        pass();
                    } else {
                        fail('Unable to find checkbox item ' + checkboxUidFormula + ' on checkbox list ' + fieldName + '.');
                    }
                }
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['assertfieldexists'] = function (pass, fail, parameters, fieldName) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field) {
                pass();
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );

}
AppCore.SML.Commands['assertfieldnotexists'] = function (pass, fail, parameters, fieldName) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (!field) {
                pass();
            } else {
                fail('Field ' + fieldName + ' exists, but should not.');
            }
        },
        function () { pass(); },
        0 /* Retries */
    );
}
AppCore.SML.Commands['assertfieldenabled'] = function (pass, fail, parameters, fieldName) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field && field.IsEnabled() === true) {
                pass();
            } else if (field) {
                fail('Field ' + fieldName + ' not enabled.');
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['assertfielddisabled'] = function (pass, fail, parameters, fieldName) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field && !field.IsEnabled() === true) {
                pass();
            } else if (field) {
                fail('Field ' + fieldName + ' not disabled.');
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['assertfieldvalid'] = function (pass, fail, parameters, fieldName) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () {
            if (PageCore.Model.CurrentPage().Form().WaitListManager.TotalCounter == 0)
            {
                return field = AppCore.SML.Helpers.FindFieldByPath(fieldName);
            }
        },
        function () {
            if (field && field.IsValid() === true) {
                pass();
            } else if (field) {
                fail('Field ' + fieldName + ' not valid.');
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['assertfieldinvalid'] = function (pass, fail, parameters, fieldName) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () {
            if (PageCore.Model.CurrentPage().Form().WaitListManager.TotalCounter == 0)
            {
                return field = AppCore.SML.Helpers.FindFieldByPath(fieldName);
            }
        },
        function () {
            if (field && !field.IsValid() === true) {
                pass();
            } else if (field) {
                fail('Field ' + fieldName + ' not invalid.');
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['assertfieldvalue'] = function (pass, fail, parameters, fieldName, fieldValue) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field.Value() == fieldValue) {
                pass();
            } else if (field) {
                fail('Field value ' + field.Value() + ' does not match expectation ' + fieldValue + '.');
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['assertfielduomvalue'] = function (pass, fail, parameters, fieldName, fieldUomValue) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field.UomValue() == fieldUomValue) {
                pass();
            } else if (field) {
                fail('Field UOM value ' + field.UomValue() + ' does not match expectation ' + fieldUomValue + '.');
            } else {
                fail('Unable to find field ' + fieldName + '.');
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['assertcheckboxstate'] = function (pass, fail, parameters, fieldName) {
    var field = null;
    AppCore.SML.Helpers.waitUntilDone(
        function () { return field = AppCore.SML.Helpers.FindFieldByPath(fieldName); },
        function () {
            if (field.FieldDef.FieldType == 'HierarchyPicker') {
                // MultiHierarchyPicker
                var checkboxUids = parameters.splice(1).sort().join(",");
                var visibleCheckboxUids = field.FullValue().Value.map(function (item) { return item.Item }).sort().join(",");
                if (checkboxUids == visibleCheckboxUids) {
                    pass();
                } else {
                    fail('Marked business entities ' + visibleCheckboxUids + ' do not match the expected business entities ' + checkboxUids);
                }
            } else {
                // CheckBoxList and potentially other field types
                var checkboxUids = parameters.splice(1).sort().join(",");
                var visibleCheckboxUids = field.FullValue().Rows.map(function (item) { return item.Item }).sort().join(",");
                if (checkboxUids == visibleCheckboxUids) {
                    pass();
                } else {
                    fail('Marked checkbox items ' + visibleCheckboxUids + ' do not match the expected checkbox items ' + checkboxUids);
                }
            }
        },
        function () { fail('Unable to find field ' + fieldName + '.'); }
    );
}
AppCore.SML.Commands['assertformula'] = function (pass, fail, parameters, expression) {
    var result = PageCore.Model.CurrentPage().Form().ExpressionEngine.EvalValue(PageCore.Model.CurrentPage().Form().RootRow, null, expression);
    if (result === true) {
        pass();
    } else {
        fail('The expression did not evaluate to true but to: ' + result);
    }
}
AppCore.SML.Commands['assertworkflowbuttonsvisible'] = function (pass, fail, parameters) {
    var buttonCodes = parameters.sort().join(",");
    var visibleButtonCodes = PageCore.Model.CurrentPage().Form().VisibleWorkflowButtons().map(function (button) { return button.ButtonCode }).sort().join(",");
    if (buttonCodes == visibleButtonCodes) {
        pass();
    } else {
        fail('Visible buttons ' + visibleButtonCodes + ' do not match the expected buttons ' + buttonCodes);
    }
}
AppCore.SML.Commands['assertworkflowstepsactive'] = function (pass, fail, parameters) {
    var stepCodes = parameters.sort().join(",");
    var activeStepCodes = PageCore.Model.CurrentPage().Form().BuildWorkflowStateJson().filter(function (button) { return button.State == 'Active' }).map(function (button) { return button.StepCode }).sort().join(",");
    if (stepCodes == activeStepCodes) {
        pass();
    } else {
        fail('Workflow steps ' + activeStepCodes + ' do not match the expected workflow steps ' + stepCodes);
    }
}
AppCore.SML.Commands['assertworkflowstepscomplete'] = function (pass, fail, parameters) {
    var stepCodes = parameters.sort().join(",");
    var completeStepCodes = PageCore.Model.CurrentPage().Form().BuildWorkflowStateJson().filter(function (button) { return button.State == 'Complete' }).map(function (button) { return button.StepCode }).sort().join(",");
    if (stepCodes == completeStepCodes) {
        pass();
    } else {
        fail('Workflow steps ' + completeStepCodes + ' do not match the expected workflow steps ' + stepCodes);
    }
}
AppCore.SML.Commands['assertworkflowstepsinactive'] = function (pass, fail, parameters) {
    var stepCodes = parameters.sort().join(",");
    var inactiveStepCodes = PageCore.Model.CurrentPage().Form().BuildWorkflowStateJson().filter(function (button) { return button.State == 'Inactive' }).map(function (button) { return button.StepCode }).sort().join(",");
    if (stepCodes == inactiveStepCodes) {
        pass();
    } else {
        fail('Workflow steps ' + inactiveStepCodes + ' do not match the expected workflow steps ' + stepCodes);
    }
}
AppCore.SML.Commands['discardvalidationmodal'] = function (pass, fail, parameters) {
    // Check if a validation modal is showing and if not, throw an error
    if (!CommonCore.ModalHelper.IsModalVisible()) {
        fail('No validation modal was open.');
    } else {
        // Close modal
        CommonCore.ModalHelper.HideModal();
        // Wait for modal to be closed
        AppCore.SML.Helpers.waitUntilDone(
            function () { return !CommonCore.ModalHelper.IsModalVisible() },
            pass,
            function () { fail('Unable to close modal.'); }
        );
    }
}
AppCore.SML.Commands['cancelworkflowpromptmodal'] = function (pass, fail, parameters) {
    // Check if a workflow prompt modal is showing and if not, throw an error
    if (!CommonCore.ModalHelper.IsModalVisible()) {
        fail('No workflow prompt modal was open.');
    } else {
        // Close modal
        CommonCore.ModalHelper.HideModal();
        // Wait for modal to be closed
        AppCore.SML.Helpers.waitUntilDone(
            function () { return !CommonCore.ModalHelper.IsModalVisible() },
            pass,
            function () { fail('Unable to close modal.'); }
        );
    }
}
AppCore.SML.Commands['confirmworkflowpromptmodal'] = function (pass, fail, parameters) {
    // Check if a workflow prompt modal is showing and if not, throw an error
    if (!CommonCore.ModalHelper.IsModalVisible() || typeof CommonCore.ModalHelper.Model.OKButtonClick == 'undefined') {
        fail('No workflow prompt modal was open.');
    } else {
        // Click OK
        CommonCore.ModalHelper.Model.OKButtonClick();
        // Wait for modal to be closed
        AppCore.SML.Helpers.waitUntilDone(
            function () { return !CommonCore.ModalHelper.IsModalVisible() },
            pass,
            function () { fail('Unable to close modal.'); }
        );
    }
}
AppCore.SML.Commands['setworkflowpromptmodalcomments'] = function (pass, fail, parameters, value) {
    // Check if a workflow prompt modal is showing and if not, throw an error
    if (!CommonCore.ModalHelper.IsModalVisible()) {
        fail('No workflow prompt modal was open.');
    } else {
        if (typeof CommonCore.ModalHelper.Model.AllowComments != "undefined" && CommonCore.ModalHelper.Model.AllowComments() == 1) {
            // Set value
            CommonCore.ModalHelper.Model.Value(value);
            pass();
        } else {
            fail("Workflow prompt modal does not allow comments.");
        }
    }
}
AppCore.SML.Commands['fillrequireddata'] = function (pass, fail, parameters) {
    PageCore.Model.CurrentPage().Form().DebugFillRandomData({ });
    setTimeout(function () { pass() }, 2000);
}
AppCore.SML.Commands['fillrequiredrandomdata'] = function (pass, fail, parameters) {
    PageCore.Model.CurrentPage().Form().DebugFillRandomData({ RequiredOnly: 1 });
    setTimeout(function () { pass() }, 2000);
}
AppCore.SML.Commands['pause'] = function (pass, fail, parameters, seconds) {
    setTimeout(function () { pass() }, seconds * 1000);
}
AppCore.SML.Commands['breakpoint'] = function (pass, fail, parameters) {
    // Empty placeholder, holds execution when running a test using the Test Editor
    if (AppCore.SML.Status.tests[0].group == 'Ad Hoc' && AppCore.SML.Status.state == 'running') {
        AppCore.SML.Status.state = "paused";
        return;
    }
    pass();
}
AppCore.SML.Commands['stop'] = function (pass, fail, parameters) {
    fail('Stop command received.');
}
AppCore.SML.Commands['clearfeatureflags'] = function (pass, fail, parameters) {
    CommonCore.PostData('sml/clearfeatureflags', {}, function (data) {
        var o = CommonCore.ParseJson(data);
        if (o.ResultCode === 'OK') {
            // Trigger a reload of feature flags
            AppCore.LoadFeatureFlagCache(function () {
                pass();
            }, function (message) {
                fail('Client error while clearing flags: ' + message);
            });
        } else {
            fail('Server error while clearing flags: ' + o.Message + '-' + o.Details);
        }
    }, function (message) {
        fail('Connection error while clearing flags: ' + message);
    });
}
AppCore.SML.Commands['setfeatureflag'] = function (pass, fail, parameters, flagKey, flagValue) {
    var json = { FlagKey: flagKey, FlagValue: flagValue };
    CommonCore.PostData('sml/setfeatureflag', ko.toJSON(json), function (data) {
        var o = CommonCore.ParseJson(data);
        if (o.ResultCode === 'OK') {
            // Trigger a reload of feature flags
            AppCore.LoadFeatureFlagCache(function () {
                pass();
            }, function (message) {
                fail('Client error while updating flags: ' + message);
            });
        } else {
            fail('Server error while updating flags: ' + o.Message + '-' + o.Details);
        }
    }, function (message) {
        fail('Connection error while updating flags: ' + message);
    });
}
AppCore.SML.Commands['revokealluserroles'] = function (pass, fail, parameters) {
    CommonCore.PostData('sml/revokealluserroles', ko.toJSON({ UserUID: AppCore.UserContext().UserUID }), function (data) {
        var o = CommonCore.ParseJson(data);
        if (o.ResultCode === 'OK') {
            // Trigger a reload of user context to make sure role changes get applied
            AppCore.LoadUserContext(function () {
                pass();
            }, function (message) {
                fail('Client error while revoking user roles: ' + message);
            }, true);
        } else {
            fail('Server error while revoking user roles: ' + o.Message + '-' + o.Details);
        }
    }, function (message) {
        fail('Connection error while revoking user roles: ' + message);
    });
}
AppCore.SML.Commands['assignalluserroles'] = function (pass, fail, parameters) {
    CommonCore.PostData('sml/assignalluserroles', ko.toJSON({ UserUID: AppCore.UserContext().UserUID }), function (data) {
        var o = CommonCore.ParseJson(data);
        if (o.ResultCode === 'OK') {
            // Trigger a reload of user context to make sure role changes get applied
            AppCore.LoadUserContext(function () {
                pass();
            }, function (message) {
                fail('Client error while assigning user roles: ' + message);
            }, true);
        } else {
            fail('Server error while assigning user roles: ' + o.Message + '-' + o.Details);
        }
    }, function (message) {
        fail('Connection error while assigning user roles: ' + message);
    });
}
AppCore.SML.Commands['assignuserrole'] = function (pass, fail, parameters, roleCode, businessEntityTitle) {
    var body = {
        UserUID: AppCore.UserContext().UserUID,
        RoleCode: roleCode,
        BusinessEntityTitle: businessEntityTitle
    }
    CommonCore.PostData('sml/assignuserrole', ko.toJSON(body), function (data) {
        var o = CommonCore.ParseJson(data);
        if (o.ResultCode === 'OK') {
            // Trigger a reload of user context to make sure role changes get applied
            AppCore.LoadUserContext(function () {
                pass();
            }, function (message) {
                fail('Client error while assigning user role: ' + message);
            }, true);
        } else {
            fail('Server error while assigning user role: ' + o.Message + '-' + o.Details);
        }
    }, function (message) {
        fail('Connection error while assigning user role: ' + message);
    });
}
AppCore.SML.Commands['revokeuserrole'] = function (pass, fail, parameters, roleCode) {
    var body = {
        UserUID: AppCore.UserContext().UserUID,
        RoleCode: roleCode
    }
    CommonCore.PostData('sml/revokeuserrole', ko.toJSON(body), function (data) {
        var o = CommonCore.ParseJson(data);
        if (o.ResultCode === 'OK') {
            // Trigger a reload of user context to make sure role changes get applied
            AppCore.LoadUserContext(function () {
                pass();
            }, function (message) {
                fail('Client error while revoking user role: ' + message);
            }, true);
        } else {
            fail('Server error while revoking user role: ' + o.Message + '-' + o.Details);
        }
    }, function (message) {
        fail('Connection error while revoking user role: ' + message);
    });
}


/* ----------------------------------------------------------
    * 
    * Editor
    * 
    ---------------------------------------------------------- */

AppCore.SML.Editor = AppCore.SML.Editor || {};

AppCore.SML.Editor.Open = function () {
    CommonCore.log("AppCore.SML.Editor.Open");
    AppCore.SML.Editor.PopupWindow = window.open('javascript:void();', 'smleditorpopup', 'width=700, height=600, location=0, scrollbars=no');
    AppCore.SML.Editor.PopupWindow.document.close();
    var html = '';
    html += '<html>';
    html += '  <head>';
    html += '    <title>EHS Insight - SML Test Editor</title>';
    html += '    <link rel="stylesheet" href="/content/bootstrap/css/bootstrap.min.css">';
    html += '    <link rel="stylesheet" href="/content/fontawesome/css/font-awesome.css">';
    html += '    <link rel="stylesheet" href="/content/codemirror/lib/codemirror.css">';
    html += '    <script src="/content/codemirror/lib/codemirror.js"></script>';
    html += '    <script src="/content/codemirror/mode/sml/sml.js"></script>';
    html += '    <style>';
    html += '      .current-line { background-color: #fff3b0; }';
    html += '    </style>';
    html += '  </head>';
    html += '  <body style="margin: 0px; overflow: none;">';
    html += '    <div style="position: absolute; padding: 10px;">';
    html += '      <button id="buttonRun"        class="btn btn-success"  title="Run" onclick="window.opener.AppCore.SML.Editor.Run(editor.getValue());"><i class="fa fa-play" aria-hidden="true"></i></button> ';
    html += '      <button id="buttonSingleStep" class="btn btn-success"  title="Single Step" onclick="window.opener.AppCore.SML.Editor.NextStep(editor.getValue());"><i class="fa fa-step-forward" aria-hidden="true"></i></button> ';
    html += '      <button id="buttonPause"      class="btn btn-success"  title="Pause" onclick="window.opener.AppCore.SML.Editor.Pause();"><i class="fa fa-pause" aria-hidden="true"></i></button> ';
    html += '      <button id="buttonActiveLine" class="btn btn-warning"  title="Set active line" onclick="window.opener.AppCore.SML.Editor.SetActiveLine(window.prompt(\'Enter the line number from which you want to continue test execution.\', window.opener.AppCore.SML.Status.commandID + 1));"><i class="fa fa-arrows-v" aria-hidden="true"></i></button> ';
    html += '      <button id="buttonStop"       class="btn btn-danger"   title="Stop" onclick="window.opener.AppCore.SML.Editor.Stop();"><i class="fa fa-stop" aria-hidden="true"></i></button> ';
    html += '      <button id="buttonLibrary"    class="btn btn-secondary" title="Open SML Test Library" onclick="window.opener.location.hash = \'/smltestlibrary\';"><i class="fa fa-list-ul" aria-hidden="true"></i></button> ';
    html += '    </div>';
    html += '    <div style="position: absolute; top: 50px; bottom: 0px;">';
    html += '    <textarea id="script">';
    html +=        CommonCore.GetLocalStorageValue('AppCore_SML_EditorValue');
    html += '    </textarea>';
    html += '    </div>';
    html += '    <script>';
    html += '      var editor = CodeMirror.fromTextArea(document.getElementById("script"), { mode: "sml", lineNumbers: true, lineWrapping: true });';
    html += '      editor.setSize("100%", "100%");';
    html += '      var currentLine = 0;';
    html += '      setInterval(function () {';
    html += '        /* Button toggles */';
    html += '        if (window.opener.AppCore.SML.Status.state == "running") { buttonRun.disabled = true;  buttonPause.disabled = false; buttonSingleStep.disabled = true;  buttonActiveLine.disabled = true;  buttonStop.disabled = false; }';
    html += '        if (window.opener.AppCore.SML.Status.state == "paused")  { buttonRun.disabled = false; buttonPause.disabled = true;  buttonSingleStep.disabled = false; buttonActiveLine.disabled = false; buttonStop.disabled = false; }';
    html += '        if (window.opener.AppCore.SML.Status.state == "stopped") { buttonRun.disabled = false; buttonPause.disabled = true;  buttonSingleStep.disabled = true;  buttonActiveLine.disabled = true;  buttonStop.disabled = true; }';
    html += '        if (window.opener.AppCore.SML.Status.state == "error")   { buttonRun.disabled = false; buttonPause.disabled = true;  buttonSingleStep.disabled = false; buttonActiveLine.disabled = false; buttonStop.disabled = false; }';
    html += '        /* Highlight current line */';
    html += '        if (typeof window.opener.AppCore.SML.Status.tests == "object" && window.opener.AppCore.SML.Status.tests !== null) {';
    html += '          if (window.opener.AppCore.SML.Status.tests[0].group == "Ad Hoc") {';
    html += '            editor.removeLineClass(currentLine, "background", "current-line");';
    html += '            currentLine = Math.min(window.opener.AppCore.SML.Status.commandID, editor.lineCount() - 1);'
    html += '            editor.addLineClass(currentLine, "background", "current-line");';
    html += '          }';
    html += '        }';
    html += '      }, 300);';
    html += '    </script>';
    html += '  </body>';
    html += '</html>';
    AppCore.SML.Editor.PopupWindow.document.write(html);
};

AppCore.SML.Editor.Run = function (sml) {
    CommonCore.log("AppCore.SML.Editor.Run");
    CommonCore.SetLocalStorageValue("AppCore_SML_EditorValue", sml, false);
    var script = AppCore.SML.Helpers.ParseSML("SML Test Editor", "Ad Hoc", sml);
    // If script was previously stopped, restart the script. Otherwise, update the script and continu where left of.
    if (AppCore.SML.Status.state == 'stopped') {
        AppCore.SML.Tests.Start([script]);
    } else {
        AppCore.SML.Status.tests = [script];
        AppCore.SML.Status.state = 'running';
        AppCore.SML.Test.NextCommand();
    }
}
AppCore.SML.Editor.NextStep = function (sml) {
    CommonCore.log("AppCore.SML.Editor.NextStep");
    CommonCore.SetLocalStorageValue("AppCore_SML_EditorValue", sml, false);
    var script = AppCore.SML.Helpers.ParseSML("SML Test Editor", "Ad Hoc", sml);
    // Update script and run next command
    AppCore.SML.Status.tests = [script];
    AppCore.SML.Status.state = 'singlestep';
    AppCore.SML.Status.commandID--;
    AppCore.SML.Test.NextCommand();
}
AppCore.SML.Editor.SetActiveLine = function (line) {
    if (line) {
        AppCore.SML.Status.commandID = line - 1;
    }
}
AppCore.SML.Editor.Pause = function () {
    CommonCore.log("AppCore.SML.Editor.Pause");
    AppCore.SML.Status.state = 'paused';
}
AppCore.SML.Editor.Stop = function () {
    CommonCore.log("AppCore.SML.Editor.Stop");
    AppCore.SML.Status.state = 'stopped';
    AppCore.SML.Test.Done();
}


/* ----------------------------------------------------------
    * 
    * Helpers
    * 
    ---------------------------------------------------------- */


AppCore.SML.Helpers = AppCore.SML.Helpers || {};

AppCore.SML.Helpers.ParseSML = function (testName, groupName, sml) {
    /*
        * Parse the SML syntax into the right JSON structure.
        * Example:
        {
        "name": "Name of test",
        "group": "Name of group",
        "commands":
        [
            {
                "command": "AddNewForm", 
                "parameters": ["CAPA", "...", "..."]
            }
        ]
        }
    */
    var script = {};
    script.name = testName;
    script.group = groupName;
    script.commands = sml.split("\n");
    script.commands = script.commands.map(function (line) {

        var command;
        var parameters = [];

        if (CommonCore.StartsWith(line.trim(), "#") || line.trim().length == 0) {
            command = "Comment";
        } else {
            command = CommonCore.StringSplitTrim(line, ",")[0]; // First CSV column
            var parameterCsv = line.trim().substring(command.length + 1) + ","; // Get the csv of all parameters , add extra comma at the end so parser knows where to stop and output the last parameter.
            var parameter = "";
            var i = 0;

            while (i < parameterCsv.length) {
                // Check current character
                switch (parameterCsv.substr(i, 1)) {
                    // Output value of parameter when encountering an unescaped comma
                    case ',':
                        parameters.push(parameter);
                        parameter = "";
                        break;
                    // Encountered a quote or parentheses escape character. Add all characters following it to parameter until the closing escape character is found.
                    case '\'':
                    case '"':
                    case '(':
                        var singleQuotes = false;
                        var doubleQuotes = false;
                        var parentheses = 0;
                        do {
                            parameter += parameterCsv.substr(i, 1);
                            switch (parameterCsv.substr(i, 1)) {
                                case '\'':
                                    singleQuotes = !singleQuotes;
                                    break;
                                case '"':
                                    doubleQuotes = !doubleQuotes;
                                    break;
                                case '(':
                                    parentheses++;
                                    break;
                                case ')':
                                    parentheses--;
                                    break;
                            }
                            i++;
                        } while (doubleQuotes == true || singleQuotes == true || parentheses > 0)
                        i--;
                        break;
                    // Add current character to parameter when it's not a quote or parentheses
                    default:
                        parameter += parameterCsv.substr(i, 1);
                        break;
                }
                i++;
            }
        }
        return {
            "command": command,
            "parameters": parameters
        }
    });
    return script;
}

AppCore.SML.Helpers.FindFieldByPath = function (fieldNamePath) {
    // Takes a path expression like one of the below and returns the field object:
    // - BusinessEntity
    // - InjuryIllnessConsequence.Row(0).FirstName
    // - InjuryIllnessConsequence.Row(0).LostTime.Row(1).StartDate

    CommonCore.log('Workflow - FindFieldByPath - ' + fieldNamePath);

    var tree = jsep(fieldNamePath);
    var fields = AppCore.SML.Helpers.ParseJsep(tree);

    var currentRow = PageCore.Model.CurrentPage().Form().RootRow;
    var field = null;

    for (var i = 0; i < fields.length; i++) {
        if (i < fields.length - 1) {
            // Grid row
            currentRow = currentRow.FindField(fields[i].fieldName).ChildRows()[fields[i].rowId];
            if (currentRow == null) {
                throw new Error('Field not found: ' + fieldNamePath);
            }
        } else {
            // Field
            field = currentRow.FindField(fields[i].fieldName);
            if (field == null) {
                throw new Error('Field not found: ' + fieldNamePath);
            }
        }
    }
    return field;
};

AppCore.SML.Helpers.ParseJsep = function (tree, rowId) {
    var path = [];
    switch (tree.type) {
        case 'Identifier':
            path = path.concat([{ fieldName: tree.name, rowId: rowId }]);
            break;
        case 'MemberExpression':
            path = path.concat(AppCore.SML.Helpers.ParseJsep(tree.object));
            path = path.concat(AppCore.SML.Helpers.ParseJsep(tree.property, rowId));
            break;
        case 'CallExpression':
            path = path.concat(AppCore.SML.Helpers.ParseJsep(tree.callee.object, tree.arguments[0].value));
            break;
        default:
            throw new Error('ExpressionEngine.ParseJsep - Unknown tree.type: ' + tree.type);
    }
    return path;
};

AppCore.SML.Helpers.waitUntilDone = function (expression, callbackPositive, callbackNegative, retryCounter) {
    if (typeof retryCounter == "undefined") {
        retryCounter = 30
    }
    // Waits for function "expression" to evaluate to true within retryCounter tries.
    var result;
    try {
        result = expression();
    }
    catch (e) {
        // Purposefully discarding error
    }
    finally {
        if (result) {
            callbackPositive();
        }
        else if (retryCounter >= 0) {
            retryCounter--;
            setTimeout(function () {
                AppCore.SML.Helpers.waitUntilDone(expression, callbackPositive, callbackNegative, retryCounter);
            }, 1000);
        }
        else {
            callbackNegative();
        }
    }
}