"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var CommonCore = CommonCore || {};
CommonCore.RandomIntFromInterval = function (min, max) {
    return Math.floor(Math.random() * ( max - min + 1) + min);  //  http://stackoverflow.com/questions/4959975/generate-random-value-between-two-numbers-in-javascript
};

MForm.prototype.DebugFillRandomData = function (options) {
    var self = this;

    if (self.Manifest.EntityName == 'IndustrialHygiene' && options.RequiredOnly != 1) {
        options.RequiredOnly = 1;
    }
    self.RootRow.DebugFillRandomData(options);
};

MRow.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    for (var i = 0; i < self.Fields.length; i++) {
        if (self.Fields[i].DebugFillRandomData !== undefined) {
            self.Fields[i].DebugFillRandomData(opt);
        }
    }
};

MCheckBox.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    var index = CommonCore.RandomIntFromInterval(0, 20);
    if (index > 10) {
        self.Value('1');
    }
    else if (index) {
        self.Value('0');
    }
};

MCheckBoxList.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    if (self.FieldDef.FieldName=='Consequences') {
        for (var i = 0; i < self.OptionsList().length; i++) {
            self.OptionsList()[i].IsChecked(true);
        }
    }
    else if (self.OptionsList().length > 0) {
        var itemCount = 1;
        var optionCount = self.OptionsList().length;
        if (optionCount > 20) {
            itemCount = 4;
        }
        else if (optionCount > 10) {
            itemCount = 3;
        }
        else if (optionCount > 5) {
            itemCount = 2;
        }

        for (var i = 0; i < itemCount; i++) {
            var index = CommonCore.RandomIntFromInterval(0, self.OptionsList().length - 1);
            self.OptionsList()[index].IsChecked(true);
        }
    }
};

MDatePicker.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    var days = 0; // Default to Today

    if (opt.FieldOverrides !== undefined && opt.FieldOverrides[self.FieldDef.FieldName] !== undefined) {
        var myopt = opt.FieldOverrides[self.FieldDef.FieldName];
        days = CommonCore.RandomIntFromInterval(myopt.Min, myopt.Max);
    }

    var newValue = moment().subtract(days, 'days').format('YYYY-MM-DD');
    self.Value(newValue);
};

MDropDown.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    if (self.FilteredDisplayOptionsList().length >= 1) {
        var index = CommonCore.RandomIntFromInterval(1, self.FilteredDisplayOptionsList().length - 1);
        // Do not select "Other" option
        if (self.FieldDef.AllowOther == 1 && index == self.FilteredDisplayOptionsList().length - 1) {
            index--;
        }
        // Do not select "Not Yet Classified" option on IncidentEvent classification
        if (['Classification', 'EnvironmentalClassification', 'VirtualClassification', 'PhysicalClassification'].includes(self.FieldDef.FieldName) && index == 1) {
            index = 2;
        }
        // If there is just one item, select that
        if (self.FilteredDisplayOptionsList().length == 1) {
            index = 0;
        }
        var selItem = self.FilteredDisplayOptionsList()[index];
        self.Value(selItem.RowUID);
    }
};

MRadioButtonList.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0) return;

    var index = CommonCore.RandomIntFromInterval(1, self.FilteredFullList().length - 1);
    var selItem = self.FilteredFullList()[index];
    self.Value(selItem.RowUID);
}

MSignature.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0) return;

    self.Value(self.FieldDef.FieldName + " signature");
};

MQuestionnaireSelector.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    if (self.OptionsList().length > 1) {
        var index = CommonCore.RandomIntFromInterval(1, self.OptionsList().length - 1);
        var selItem = self.OptionsList()[index];
        self.Value(selItem.RowUID);
    }
};


MGrid.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    switch (self.FieldDef.FieldName) {
        case 'WorkflowHistory':
        case 'WorkflowState':
        case 'WorkflowTasks':
        case 'WorkflowEmails':
        case 'LaminationHistory':
        case 'ExposureAssessment': // Grid on IndustrialHygiene that is auto generated by a DataSourceIHAssessmentPerAgentGroup field
            //  skipping grids that do not need random data..
            break;

        default:

            if (opt.RequiredOnly != 1 || self.ValidationMessages().length > 0) {

                if (self.ChildRows().length === 0) {
                    var cnt = self.ParentForm.ExpressionEngine.EvalValue(self.Row, null, self.FieldDef.DefaultNewRowCountFormula);
                    if (CommonCore.IsNullOrEmpty(cnt) || cnt == 0) {
                        cnt = CommonCore.RandomIntFromInterval(1, 5);
                    }

                    if (CommonCore.IsInt32(cnt)) {
                        for (var i = 0; i < CommonCore.ParseInt32(cnt); i++) {
                            self.AddRow(null, true);
                        }
                    }
                }

            }

            for (var j = 0; j < self.ChildRows().length; j++) {
                var row = self.ChildRows()[j];
                row.DebugFillRandomData(opt);
            }

            break;
    }
};


MQuestionnaire.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    var fillQuestionnaire = function () {
        if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0) return;

        var i, j, k, group, section, question;

        for (i = 0; group = self.Groups()[i]; i++) {
            for (j = 0; section = group.Sections()[j]; j++) {
                for (k = 0; question = section.Questions()[k]; k++) {
                    question.AnswerValue(1);
                }
            }
        }
        for (j = 0; section = self.Sections()[j]; j++) {
            for (k = 0; question = section.Questions()[k]; k++) {
                question.AnswerValue(1);
            }
        }
        if (self.Groups().length == 0 || self.Sections() == 0) {
            // If the questions are not yet loaded, back out and try again in one second
            window.setTimeout(function () {
                fillQuestionnaire();
            }, 1000);
        }
    }
    fillQuestionnaire();
};


MHierarchyPicker.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    //  todo: [future] refactor: currently this picks any random node in the tree as its selection.. it needs to be updated to pick only selectable nodes.. this will require refactoring some logic out of the ModalCore.HierarchyPickerModal as it currently has all the logic to
    //         to build of the object graph and to determine what node can be selected and tree node visibility.

    if (self.FullList().length > 0) {
        var effectiveFullList = [];
        for (var i = 0; i < self.FullList().length; i++) {
            var row = self.FullList()[i].Row;
            if (row.StartDate <= self.EffectiveDate() && row.EndDate > self.EffectiveDate()) {
                effectiveFullList.push(row);
            }
        }

        var selIndex = CommonCore.RandomIntFromInterval(0, effectiveFullList.length - 1);
        var selItem = effectiveFullList[selIndex];
        self.Value(selItem.RowUID);
    }
};

MTextBox.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    var min = 0, max = 100;

    if (opt.FieldOverrides !== undefined && opt.FieldOverrides[self.FieldDef.FieldName] !== undefined) {
        var myopt = opt.FieldOverrides[self.FieldDef.FieldName];
        min = myopt.Min;
        max = myopt.Max;
    }

    var newValue = '';
    switch (self.FieldDef.DataType) {
        case 'Text':
        case 'String':
            newValue = this.FieldDef.FieldName + ' ' + moment().format('hhmmss');
            if (self.FieldDef.FieldName == 'EmailAddress') {
                newValue = newValue.replace(' ','') + '@sts.local';
            }
            break;

        case 'Integer':
            newValue = CommonCore.RandomIntFromInterval(min, max).toString();
            break;

        case 'Decimal2':
            newValue = CommonCore.RandomIntFromInterval(min, max).toString() + '.' + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString();
            break;

        case 'Decimal4':
            newValue = CommonCore.RandomIntFromInterval(min, max).toString() + '.' + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString();
            break;
    }

    if (self.FieldDef.FieldName == 'Year') { // Specifically for Objective
        newValue = new Date().getFullYear();
    }

    self.Value(newValue);
};

MTextBoxWithUom.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    //  TEXT
    var newValue = '';
    switch (self.FieldDef.DataType) {
        case 'Text':
        case 'String':
            newValue = this.FieldDef.FieldName + ' ' + moment().format('hhmmss');
            break;

        case 'Integer':
            newValue = CommonCore.RandomIntFromInterval(0, 1000).toString();
            break;

        case 'Decimal2':
            newValue = CommonCore.RandomIntFromInterval(0, 1000).toString() + '.' + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString();
            break;

        case 'Decimal4':
            newValue = CommonCore.RandomIntFromInterval(0, 1000).toString() + '.' + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString();
            break;

        case 'Decimal6':
            newValue = CommonCore.RandomIntFromInterval(0, 1000).toString() + '.' + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString() + CommonCore.RandomIntFromInterval(0, 9).toString();
            break;

    }

    self.Value(newValue);

    //  UOM DropDown
    var uomLoopCounter = 0;
    var setUomValue = function () {
        if (self.FilteredOptionsList().length > 1) {
            if (!self.HasValue()) {
                var selItem = CommonCore.RandomIntFromInterval(1, self.FilteredOptionsList().length - 1);
                self.UomValue(self.FilteredOptionsList()[selItem].RowUID);
            }
        }
        window.setTimeout(function () {
            // Since the UOM options might change based on other field values, use a loop to ensure for at least the next 10 seconds we are always trying to resolve to a new value if there is a change.
            if (uomLoopCounter < 10) {
                setUomValue();
            }
            uomLoopCounter++
        }, 1000);
    }
    setUomValue();
};

MTimePicker.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    var minutes = CommonCore.RandomIntFromInterval(0, 1440);
    if (self.FieldDef.IncludeSeconds == 1) {
        self.Value(moment().subtract(minutes, 'minutes').format('HH:mm:ss'));
    }
    else {
        self.Value(moment().subtract(minutes, 'minutes').format('HH:mm'));
    }
};

MRiskMatrix.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    self.Value('10');
    self.DataValue('1A');
    self.ScoreValue('1');
};

MYesNo.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    if ((self.FieldDef.FieldName == 'IsSingleAssignment') || (self.FieldDef.FieldName == 'IsCompanyAsset')) {
        self.Value('1');
    } else {
        self.Value('0'); // Select No by default
    }
};


MSinglePicker.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    var name = self.FieldDef.FieldName;

    if (opt === undefined || opt[name] === undefined || opt[name].Value === undefined) {
        //  no override was provided, but just in case this single picker is configured against InstanceUser table then lets use a random hardcoded value taken from the InstanceUser table at random
        if ((self.FieldDef.LookupEntityName == 'InstanceUser') || (self.FieldDef.LookupEntityName == 'UserContact')) {
            var names = ['dave elper'];
            var index = CommonCore.RandomIntFromInterval(0, 0);
            self.Text(names[index]);
            self.Resolve();
        } else {
            self.Text('a');
            self.DebugDiagnosticFill = true; // Required to make MSinglePicker add resolved values to self.DebugDiagnosticFillResults
            self.DebugDiagnosticFillResults = [];
            self.Resolve();
            var MSinglePickerResolve = function() {
                var results = self.DebugDiagnosticFillResults;
                if (results.length > 0) {
                    var selItem = CommonCore.RandomIntFromInterval(0, results.length - 1);
                    
                    var searchField = 'Title';
                    var fields = self.ParsePopupFields(); // Not every MSinglePicker uses the Title field to search
                    if (fields.SearchFields) {
                        searchField = fields.SearchFields[0];
                    }

                    self.Text(results[selItem][searchField]);
                    self.Resolve();
                    self.DebugDiagnosticFill = false;
                } else {
                    window.setTimeout(MSinglePickerResolve, 500);
                }
            };
            // Wait until first results are returned
            window.setTimeout(MSinglePickerResolve, 500);
        }

        return;
    }
    self.Text(opt[self.FieldName].Value);
    self.Resolve();
};

MMultiPicker.prototype.DebugFillRandomData = function (opt) {
    //  [future] V2 - if needed

};

MTreeViewPicker.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0) return;

    //self.Initialize();
    self.Text('a');
    self.DebugDiagnosticFill = true; // Required to make MSinglePicker add resolved values to self.DebugDiagnosticFillResults
    self.DebugDiagnosticFillResults = [];
    self.Resolve();

    var MTreeViewPickerResolve = function () {
        var results = self.DebugDiagnosticFillResults;
        if (results.length > 0) {
            var selItem = CommonCore.RandomIntFromInterval(0, results.length - 1);
            var searchField = self.FieldDef.LookupEntityCaption;
            self.Text(results[selItem][searchField]);
            self.Resolve();
            self.DebugDiagnosticFill = false;
        } else {
            window.setTimeout(MTreeViewPickerResolve, 500);
        }
    };
    // Wait until first results are returned
    window.setTimeout(MTreeViewPickerResolve, 500);
};

MQuestionnaireOrRouteSelector.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    var selItem = CommonCore.RandomIntFromInterval(1, self.FilteredFullList().length - 1);
    self.Value(self.FilteredFullList()[selItem].RowUID);
};

MAttachment.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0)  return;

    self.FullValue({
        AttachmentUID: 'ab0a4a6c-fb59-4c94-948d-7d61886d5249', // Exists in CorpDemo and ContosoTrial
        FileSize: 14435,
        ContentType: 'image/png',
        Value: 'Gas Cylinder.png'
    });
};

MDocumentPicker.prototype.DebugFillRandomData = function (opt) {
    var self = this;

    if (opt.RequiredOnly == 1 && self.ValidationMessages().length == 0) return;

    self.Value("4a96f8d5-73b6-4cdd-98f6-1b2acc8bfa40"); // CorpDemo: Effects of Weather
    self.EntityUID("1b5206c4-94a2-4a9a-af56-260c61ef2803"); // CorpDemo: Toolbox Talk
    self.AttachmentUID("5f31da07-4f10-45d0-bb5b-cd77c1063f91");
    self.AttachmentFileName("ToolboxTalkSample_03.pdf");
};