
// https://github.com/dgbeck/jquery.autogrow-textarea

// Based off https://code.google.com/p/gaequery/source/browse/trunk/src/static/scripts/jquery.autogrow-textarea.js?r=2
// Modified by David Beck

// MIT License

// Updated by Eric Stevens (to add +30 padding on line 85 and to add width/scroll enhancements on 75-80 and to convert to 'self' pattern)

// Updated by eric stevens for 20 min height

(function($) {

    $.fn.autogrow = function(options) {

        options = $.extend( {
            vertical: true,
            horizontal: false
        }, options);


        this.filter('textarea').each(function() {

            var self       = this,
                minHeight   = $(self).height(),
                maxHeight	= $(self).attr( "maxHeight" ),
                lineHeight  = $(self).css('lineHeight'),
                minWidth	= typeof( $(self).attr( "minWidth" ) ) == "undefined" ? 0 : $(self).attr( "minWidth" );

            if( typeof( maxHeight ) == "undefined" ) maxHeight = 1000000;

            var shadow = $('<div class="autogrow-shadow"></div>').css({
                position:   'absolute',
                top:        -10000,
                left:       -10000,
                fontSize:   $(self).css('fontSize'),
                fontFamily: $(self).css('fontFamily'),
                fontWeight: $(self).css('fontWeight'),
                lineHeight: $(self).css('lineHeight'),
                resize:     'none'
            }).appendTo(document.body);

            var update = function() {

                var times = function(string, number) {
                    for (var i = 0, r = ''; i < number; i ++) r += string;
                    return r;
                };

                var val = $(self).val();

                if( options.vertical )
                    val = val.replace(/</g, '&lt;')
                        .replace(/>/g, '&gt;')
                        .replace(/&/g, '&amp;')
                        .replace(/\n$/, '<br/>&nbsp;')
                        .replace(/\n/g, '<br/>')
                        .replace(/ {2,}/g, function(space) { return times('&nbsp;', space.length -1) + ' '; });

                //if( options.horizontal )
                //	val = $.trim( val );

                shadow.html(val).css( "width", "auto" );

                if( options.horizontal )
                {
                    var maxWidth = options.maxWidth;
                    if( typeof( maxWidth ) == "undefined" ) maxWidth = $(self).parent().width() - 12;
                    $(self).css( "width", Math.min( Math.max( shadow.width() + 9, minWidth ), maxWidth ) );
                }

                if( options.vertical )
                {
                    var curMaxHeight = maxHeight;

                    if ($(self).width() > 0 && $(self).width() < 200 && curMaxHeight == 1000000) {
                        curMaxHeight = 400;
                        $(document).one('scroll', function() {
                            update();
                        });
                    }

                    shadow.css( "width", $(self).width() - parseInt($(self).css('paddingLeft'),10) - parseInt($(self).css('paddingRight'),10) );
                    var shadowHeight = shadow.height();
                    var newHeight = Math.min( Math.max(20, Math.max( shadowHeight, minHeight )), curMaxHeight );
                    $(self).css( "height", newHeight + 30 );
                    $(self).css( "overflow", newHeight == curMaxHeight ? "auto" : "hidden" );
                }
            };

            $(this)
                .change(update)
                .keyup(update)
                .keydown(update)
                .bind( "remove.autogrow", function() {
                    shadow.remove();
                } );

            update.apply(this);

        });

        return this;
    };

})(jQuery);